# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CREATE_ICOSAHEDRON_METADATA = Metadata(
    id="e53f40ccc9791a6287b72adf8bd45877deb824ec.boutiques",
    name="CreateIcosahedron",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class CreateIcosahedronOutputs(typing.NamedTuple):
    """
    Output object returned when calling `create_icosahedron(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def create_icosahedron(
    runner: Runner | None = None,
) -> CreateIcosahedronOutputs:
    """
    Tool to create an icosahedron with optional tessellation.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/CreateIcosahedron.html
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CreateIcosahedronOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CREATE_ICOSAHEDRON_METADATA)
    cargs = []
    cargs.append("CreateIcosahedron")
    cargs.append("[-rad")
    cargs.append("r]")
    cargs.append("[-rd")
    cargs.append("recDepth]")
    cargs.append("[-ld")
    cargs.append("linDepth]")
    cargs.append("[-ctr")
    cargs.append("ctr]")
    cargs.append("[-prefix")
    cargs.append("fout]")
    cargs.append("[-min_nodes")
    cargs.append("MIN_NODES]")
    cargs.append("[-nums]")
    cargs.append("[-nums_quiet]")
    cargs.append("[-tosphere]")
    cargs.append("[-help]")
    ret = CreateIcosahedronOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CREATE_ICOSAHEDRON_METADATA",
    "CreateIcosahedronOutputs",
    "create_icosahedron",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__DJUNCT_MONTAGE_COORDINATOR_METADATA = Metadata(
    id="9334f9ba509c4429fa9b5cd5573bb0523c2d77bd.boutiques",
    name="@djunct_montage_coordinator",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VDjunctMontageCoordinatorOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__djunct_montage_coordinator(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_coords: OutputPathType
    """Output coordinates (IJK or XYZ) for the montage setup."""


def v__djunct_montage_coordinator(
    input_file: InputPathType,
    montx: float,
    monty: float,
    out_xyz: bool = False,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> VDjunctMontageCoordinatorOutputs:
    """
    Small program to calculate how to evenly space a certain number of slices within
    each view plane of a dataset.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/@djunct_montage_coordinator.html
    
    Args:
        input_file: Name of input dataset.
        montx: Montage dimension: number of panels along x-axis (i.e., number\
            of cols).
        monty: Montage dimension: number of panels along y-axis (i.e., number\
            of rows).
        out_xyz: Make program output 'X Y Z' values.
        help_: See helpfile.
        version: See version number.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VDjunctMontageCoordinatorOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__DJUNCT_MONTAGE_COORDINATOR_METADATA)
    cargs = []
    cargs.append("@djunct_montage_coordinator")
    cargs.extend([
        "-inset",
        execution.input_file(input_file)
    ])
    cargs.extend([
        "-montx",
        str(montx)
    ])
    cargs.extend([
        "-monty",
        str(monty)
    ])
    if out_xyz:
        cargs.append("-out_xyz")
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-ver")
    ret = VDjunctMontageCoordinatorOutputs(
        root=execution.output_file("."),
        output_coords=execution.output_file("output.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VDjunctMontageCoordinatorOutputs",
    "V__DJUNCT_MONTAGE_COORDINATOR_METADATA",
    "v__djunct_montage_coordinator",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

METRIC_VECTOR_TOWARD_ROI_METADATA = Metadata(
    id="685141a970a934c93282eada97a659baeaf4e5ac.boutiques",
    name="metric-vector-toward-roi",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


class MetricVectorTowardRoiOutputs(typing.NamedTuple):
    """
    Output object returned when calling `metric_vector_toward_roi(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def metric_vector_toward_roi(
    surface: InputPathType,
    target_roi: InputPathType,
    metric_out: str,
    opt_roi_roi_metric: InputPathType | None = None,
    runner: Runner | None = None,
) -> MetricVectorTowardRoiOutputs:
    """
    Find if vectors point toward an roi.
    
    At each vertex, compute the vector along the start of the shortest path to
    the ROI.
    
    Author: Washington University School of Medicin
    
    Args:
        surface: the surface to compute on.
        target_roi: the roi to find the shortest path to.
        metric_out: the output metric.
        opt_roi_roi_metric: don't compute for vertices outside an roi: the\
            region to compute inside, as a metric.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricVectorTowardRoiOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_VECTOR_TOWARD_ROI_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-metric-vector-toward-roi")
    cargs.append(execution.input_file(surface))
    cargs.append(execution.input_file(target_roi))
    cargs.append(metric_out)
    if opt_roi_roi_metric is not None:
        cargs.extend([
            "-roi",
            execution.input_file(opt_roi_roi_metric)
        ])
    ret = MetricVectorTowardRoiOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(metric_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "METRIC_VECTOR_TOWARD_ROI_METADATA",
    "MetricVectorTowardRoiOutputs",
    "metric_vector_toward_roi",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__AFNI_REFACER_MAKE_ONEBIG_A12_METADATA = Metadata(
    id="62807fe560fff2e1b847649411a05eb509a98cd8.boutiques",
    name="@afni_refacer_make_onebigA12",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VAfniRefacerMakeOnebigA12Outputs(typing.NamedTuple):
    """
    Output object returned when calling `v__afni_refacer_make_onebig_a12(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    aligned_output: OutputPathType
    """Aligned T1w dataset to MNI template with expanded 'big' grid"""


def v__afni_refacer_make_onebig_a12(
    t1w_dataset: InputPathType,
    runner: Runner | None = None,
) -> VAfniRefacerMakeOnebigA12Outputs:
    """
    Script to align a single T1w dataset to the MNI template and expand it to a
    'big' grid.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/@afni_refacer_make_onebigA12.html
    
    Args:
        t1w_dataset: Input T1w dataset name.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VAfniRefacerMakeOnebigA12Outputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__AFNI_REFACER_MAKE_ONEBIG_A12_METADATA)
    cargs = []
    cargs.append("@afni_refacer_make_onebigA12")
    cargs.append(execution.input_file(t1w_dataset))
    ret = VAfniRefacerMakeOnebigA12Outputs(
        root=execution.output_file("."),
        aligned_output=execution.output_file(pathlib.Path(t1w_dataset).name + "_aligned_to_MNI.nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VAfniRefacerMakeOnebigA12Outputs",
    "V__AFNI_REFACER_MAKE_ONEBIG_A12_METADATA",
    "v__afni_refacer_make_onebig_a12",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SERIAL_HELPER_METADATA = Metadata(
    id="74624be5b2d9de440d18ab39c983e9f94542342d.boutiques",
    name="serial_helper",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class SerialHelperOutputs(typing.NamedTuple):
    """
    Output object returned when calling `serial_helper(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def serial_helper(
    runner: Runner | None = None,
) -> SerialHelperOutputs:
    """
    Passes motion parameters from socket to serial port.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/serial_helper.html
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SerialHelperOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SERIAL_HELPER_METADATA)
    cargs = []
    cargs.append("serial_helper")
    cargs.append("[OPTIONS]")
    ret = SerialHelperOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SERIAL_HELPER_METADATA",
    "SerialHelperOutputs",
    "serial_helper",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__SUMA_REPREFIXIZE_SPEC_METADATA = Metadata(
    id="19385e693af8f09dac5faa2aadd037a6f25c79e7.boutiques",
    name="@suma_reprefixize_spec",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VSumaReprefixizeSpecOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__suma_reprefixize_spec(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    prefixed_spec_files: OutputPathType
    """Prefixed SUMA specification files"""


def v__suma_reprefixize_spec(
    input_file: InputPathType,
    prefix: str,
    output_dir: str,
    work_dir: str,
    no_clean: bool = False,
    runner: Runner | None = None,
) -> VSumaReprefixizeSpecOutputs:
    """
    A tool for prefixing and working with SUMA specification files.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/@suma_reprefixize_spec.html
    
    Args:
        input_file: Input SUMA specification file.
        prefix: Prefix to be added to the file names.
        output_dir: Output directory where the prefixed files will be saved.
        work_dir: Working directory for temporary files.
        no_clean: Flag to avoid cleaning temporary files.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VSumaReprefixizeSpecOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__SUMA_REPREFIXIZE_SPEC_METADATA)
    cargs = []
    cargs.append("suma_reprefixize_spec")
    cargs.extend([
        "-input",
        execution.input_file(input_file)
    ])
    cargs.extend([
        "-preprefix",
        prefix
    ])
    cargs.extend([
        "-odir",
        output_dir
    ])
    cargs.extend([
        "-workdir",
        work_dir
    ])
    if no_clean:
        cargs.append("-no_clean")
    ret = VSumaReprefixizeSpecOutputs(
        root=execution.output_file("."),
        prefixed_spec_files=execution.output_file(output_dir + "/*.spec"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VSumaReprefixizeSpecOutputs",
    "V__SUMA_REPREFIXIZE_SPEC_METADATA",
    "v__suma_reprefixize_spec",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

VOLUME_ALL_LABELS_TO_ROIS_METADATA = Metadata(
    id="0f6ea8b27a35d05248ce7cf9e5b3318c7359a543.boutiques",
    name="volume-all-labels-to-rois",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


class VolumeAllLabelsToRoisOutputs(typing.NamedTuple):
    """
    Output object returned when calling `volume_all_labels_to_rois(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume file"""


def volume_all_labels_to_rois(
    label_in: InputPathType,
    map_: str,
    volume_out: str,
    runner: Runner | None = None,
) -> VolumeAllLabelsToRoisOutputs:
    """
    Make rois from all labels in a volume frame.
    
    The output volume has a frame for each label in the specified input frame,
    other than the ??? label, each of which contains an ROI of all voxels that
    are set to the corresponding label.
    
    Author: Washington University School of Medicin
    
    Args:
        label_in: the input volume label file.
        map_: the number or name of the label map to use.
        volume_out: the output volume file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeAllLabelsToRoisOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_ALL_LABELS_TO_ROIS_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-volume-all-labels-to-rois")
    cargs.append(execution.input_file(label_in))
    cargs.append(map_)
    cargs.append(volume_out)
    ret = VolumeAllLabelsToRoisOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(volume_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VOLUME_ALL_LABELS_TO_ROIS_METADATA",
    "VolumeAllLabelsToRoisOutputs",
    "volume_all_labels_to_rois",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

VOLUME_LABEL_TO_SURFACE_MAPPING_METADATA = Metadata(
    id="ed347a0c13330043429633da76510254e06a655b.boutiques",
    name="volume-label-to-surface-mapping",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class VolumeLabelToSurfaceMappingRibbonConstrained:
    """
    use ribbon constrained mapping algorithm.
    """
    inner_surf: InputPathType
    """the inner surface of the ribbon"""
    outer_surf: InputPathType
    """the outer surface of the ribbon"""
    opt_volume_roi_roi_volume: InputPathType | None = None
    """use a volume roi: the volume file"""
    opt_voxel_subdiv_subdiv_num: int | None = None
    """voxel divisions while estimating voxel weights: number of subdivisions,
    default 3"""
    opt_thin_columns: bool = False
    """use non-overlapping polyhedra"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-ribbon-constrained")
        cargs.append(execution.input_file(self.inner_surf))
        cargs.append(execution.input_file(self.outer_surf))
        if self.opt_volume_roi_roi_volume is not None:
            cargs.extend([
                "-volume-roi",
                execution.input_file(self.opt_volume_roi_roi_volume)
            ])
        if self.opt_voxel_subdiv_subdiv_num is not None:
            cargs.extend([
                "-voxel-subdiv",
                str(self.opt_voxel_subdiv_subdiv_num)
            ])
        if self.opt_thin_columns:
            cargs.append("-thin-columns")
        return cargs


class VolumeLabelToSurfaceMappingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `volume_label_to_surface_mapping(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    label_out: OutputPathType
    """the output gifti label file"""


def volume_label_to_surface_mapping(
    volume: InputPathType,
    surface: InputPathType,
    label_out: str,
    ribbon_constrained: VolumeLabelToSurfaceMappingRibbonConstrained | None = None,
    opt_subvol_select_subvol: str | None = None,
    runner: Runner | None = None,
) -> VolumeLabelToSurfaceMappingOutputs:
    """
    Map a label volume to a surface label file.
    
    Map label volume data to a surface. If -ribbon-constrained is not specified,
    uses the enclosing voxel method. The ribbon mapping method constructs a
    polyhedron from the vertex's neighbors on each surface, and estimates the
    amount of this polyhedron's volume that falls inside any nearby voxels, to
    use as the weights for a popularity comparison. If -thin-columns is
    specified, the polyhedron uses the edge midpoints and triangle centroids, so
    that neighboring vertices do not have overlapping polyhedra. This may
    require increasing -voxel-subdiv to get enough samples in each voxel to
    reliably land inside these smaller polyhedra. The volume ROI is useful to
    exclude partial volume effects of voxels the surfaces pass through, and will
    cause the mapping to ignore voxels that don't have a positive value in the
    mask. The subdivision number specifies how it approximates the amount of the
    volume the polyhedron intersects, by splitting each voxel into NxNxN pieces,
    and checking whether the center of each piece is inside the polyhedron. If
    you have very large voxels, consider increasing this if you get unexpected
    unlabeled vertices in your output.
    
    Author: Washington University School of Medicin
    
    Args:
        volume: the volume to map data from.
        surface: the surface to map the data onto.
        label_out: the output gifti label file.
        ribbon_constrained: use ribbon constrained mapping algorithm.
        opt_subvol_select_subvol: select a single subvolume to map: the\
            subvolume number or name.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeLabelToSurfaceMappingOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_LABEL_TO_SURFACE_MAPPING_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-volume-label-to-surface-mapping")
    cargs.append(execution.input_file(volume))
    cargs.append(execution.input_file(surface))
    cargs.append(label_out)
    if ribbon_constrained is not None:
        cargs.extend(ribbon_constrained.run(execution))
    if opt_subvol_select_subvol is not None:
        cargs.extend([
            "-subvol-select",
            opt_subvol_select_subvol
        ])
    ret = VolumeLabelToSurfaceMappingOutputs(
        root=execution.output_file("."),
        label_out=execution.output_file(label_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VOLUME_LABEL_TO_SURFACE_MAPPING_METADATA",
    "VolumeLabelToSurfaceMappingOutputs",
    "VolumeLabelToSurfaceMappingRibbonConstrained",
    "volume_label_to_surface_mapping",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__FULL_PATH_METADATA = Metadata(
    id="55bb0db57c89c2fd7e600f678ea6686fbb3e35f7.boutiques",
    name="@FullPath",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VFullPathOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__full_path(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def v__full_path(
    input_file: InputPathType,
    runner: Runner | None = None,
) -> VFullPathOutputs:
    """
    Changes relative path to absolute one.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/@FullPath.html
    
    Args:
        input_file: Input file with relative path.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VFullPathOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__FULL_PATH_METADATA)
    cargs = []
    cargs.append("@FullPath")
    cargs.append(execution.input_file(input_file))
    ret = VFullPathOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VFullPathOutputs",
    "V__FULL_PATH_METADATA",
    "v__full_path",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

PARSE_NAME_METADATA = Metadata(
    id="a67788b0c0514057052216aadee90c72b904dbb1.boutiques",
    name="ParseName",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class ParseNameOutputs(typing.NamedTuple):
    """
    Output object returned when calling `parse_name(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def parse_name(
    runner: Runner | None = None,
) -> ParseNameOutputs:
    """
    Parses filename into components useful for AFNI.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/ParseName.html
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ParseNameOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(PARSE_NAME_METADATA)
    cargs = []
    cargs.append("ParseName")
    cargs.append("[OPTIONAL_PARAMETERS]")
    cargs.append("<FName>")
    ret = ParseNameOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "PARSE_NAME_METADATA",
    "ParseNameOutputs",
    "parse_name",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

BAYCEST_METADATA = Metadata(
    id="795753b1e84a6fd7866195155e777c5b4987a6c0.boutiques",
    name="baycest",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class BaycestOutputs(typing.NamedTuple):
    """
    Output object returned when calling `baycest(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Main output file"""


def baycest(
    data_file: InputPathType,
    mask_file: InputPathType,
    output_dir: str,
    pools_file: InputPathType,
    spec_file: InputPathType,
    ptrain_file: InputPathType,
    spatial_flag: bool = False,
    t12prior_flag: bool = False,
    runner: Runner | None = None,
) -> BaycestOutputs:
    """
    Bayesian analysis for chemical exchange saturation transfer z-spectra.
    
    Author: Bayesian Analysis Group
    
    Args:
        data_file: Specify data file (nifti image).
        mask_file: Specify mask file (nifti image).
        output_dir: Specify output directory name.
        pools_file: Specify pools to be included in model (ascii matrix).
        spec_file: Data specification (ascii matrix).
        ptrain_file: Specify pulse shape (ascii matrix).
        spatial_flag: Use spatial prior (appropriate for in vivo data).
        t12prior_flag: Include uncertainty in T1 and T2 values.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BaycestOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(BAYCEST_METADATA)
    cargs = []
    cargs.append("baycest")
    cargs.append("--data=" + execution.input_file(data_file))
    cargs.append("--mask=" + execution.input_file(mask_file))
    cargs.append("--output=" + output_dir)
    cargs.append("--pools=" + execution.input_file(pools_file))
    cargs.append("--spec=" + execution.input_file(spec_file))
    cargs.append("--ptrain=" + execution.input_file(ptrain_file))
    if spatial_flag:
        cargs.append("--spatial")
    if t12prior_flag:
        cargs.append("--t12prior")
    ret = BaycestOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(output_dir + "/output_file.nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "BAYCEST_METADATA",
    "BaycestOutputs",
    "baycest",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

RBOX_METADATA = Metadata(
    id="8ee67e27b639db12dcf69b7fcd70c8699ae8a6f6.boutiques",
    name="rbox",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class RboxOutputs(typing.NamedTuple):
    """
    Output object returned when calling `rbox(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def rbox(
    number_points: str,
    dimension: str | None = None,
    integer_coordinates: bool = False,
    bounding_box: float | None = None,
    offset: float | None = None,
    user_seed: float | None = None,
    mesh_lattice: list[str] | None = None,
    runner: Runner | None = None,
) -> RboxOutputs:
    """
    Generate various point distributions. Default is random in cube.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/rbox.html
    
    Args:
        number_points: Number of random points in cube, lens, spiral, sphere or\
            grid.
        dimension: Dimension (e.g., D3 for 3-d).
        integer_coordinates: Print integer coordinates, default 'Bn' is 1e+06.
        bounding_box: Bounding box coordinates, default 0.5.
        offset: Offset coordinates by n.
        user_seed: Use n as the random number seed.
        mesh_lattice: Lattice (Mesh) rotated by [n,-m,0], [m,n,0], [0,0,r], ...
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `RboxOutputs`).
    """
    if mesh_lattice is not None and not (3 <= len(mesh_lattice)): 
        raise ValueError(f"Length of 'mesh_lattice' must be greater than 3 but was {len(mesh_lattice)}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(RBOX_METADATA)
    cargs = []
    cargs.append("rbox")
    cargs.append(number_points)
    if dimension is not None:
        cargs.append(dimension)
    if integer_coordinates:
        cargs.append("z")
    if bounding_box is not None:
        cargs.extend([
            "B",
            str(bounding_box)
        ])
    if offset is not None:
        cargs.extend([
            "O",
            str(offset)
        ])
    if user_seed is not None:
        cargs.extend([
            "t",
            str(user_seed)
        ])
    if mesh_lattice is not None:
        cargs.extend([
            "M",
            *mesh_lattice
        ])
    ret = RboxOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "RBOX_METADATA",
    "RboxOutputs",
    "rbox",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRCOLOUR_METADATA = Metadata(
    id="13304bc23ae97d64cedc37fef80da355a36e5b22.boutiques",
    name="mrcolour",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


@dataclasses.dataclass
class MrcolourConfig:
    """
    temporarily set the value of an MRtrix config file entry.
    """
    key: str
    """temporarily set the value of an MRtrix config file entry."""
    value: str
    """temporarily set the value of an MRtrix config file entry."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-config")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


class MrcolourOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mrcolour(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image"""


def mrcolour(
    input_: InputPathType,
    map_: str,
    output: str,
    upper: float | None = None,
    lower: float | None = None,
    colour: list[float] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrcolourConfig] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrcolourOutputs:
    """
    Apply a colour map to an image.
    
    Under typical usage, this command will receive as input ad 3D greyscale
    image, and output a 4D image with 3 volumes corresponding to red-green-blue
    components; other use cases are possible, and are described in more detail
    below.
    
    By default, the command will automatically determine the maximum and minimum
    intensities of the input image, and use that information to set the upper
    and lower bounds of the applied colourmap. This behaviour can be overridden
    by manually specifying these bounds using the -upper and -lower options
    respectively.
    
    References:
    
    .
    
    Author: Robert E. Smith (robert.smith@florey.edu.au)
    
    URL:
    https://mrtrix.readthedocs.io/en/latest/reference/commands/mrcolour.html
    
    Args:
        input_: the input image.
        map_: the colourmap to apply; choices are:\
            gray,hot,cool,jet,inferno,viridis,pet,colour,rgb.
        output: the output image.
        upper: manually set the upper intensity of the colour mapping.
        lower: manually set the lower intensity of the colour mapping.
        colour: set the target colour for use of the 'colour' map (three\
            comma-separated floating-point values).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrcolourOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRCOLOUR_METADATA)
    cargs = []
    cargs.append("mrcolour")
    if upper is not None:
        cargs.extend([
            "-upper",
            str(upper)
        ])
    if lower is not None:
        cargs.extend([
            "-lower",
            str(lower)
        ])
    if colour is not None:
        cargs.extend([
            "-colour",
            ",".join(map(str, colour))
        ])
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([a for c in [s.run(execution) for s in config] for a in c])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    cargs.append(execution.input_file(input_))
    cargs.append(map_)
    cargs.append(output)
    ret = MrcolourOutputs(
        root=execution.output_file("."),
        output=execution.output_file(output),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRCOLOUR_METADATA",
    "MrcolourConfig",
    "MrcolourOutputs",
    "mrcolour",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3DTO_XDATASET_METADATA = Metadata(
    id="9011efe4c33ad7bf18deb6c5c1803fd3a072d632.boutiques",
    name="3dtoXdataset",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dtoXdatasetOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3dto_xdataset(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_sdat: OutputPathType
    """Output file in .sdat format"""


def v_3dto_xdataset(
    prefix: str,
    mask: InputPathType,
    input_files: list[InputPathType],
    runner: Runner | None = None,
) -> V3dtoXdatasetOutputs:
    """
    Convert input datasets to the format needed for 3dClustSimX.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dtoXdataset.html
    
    Args:
        prefix: Prefix for the output file.
        mask: Mask dataset file.
        input_files: Input datasets to be converted.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dtoXdatasetOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3DTO_XDATASET_METADATA)
    cargs = []
    cargs.append("3dtoXdataset")
    cargs.append("-prefix")
    cargs.append(prefix)
    cargs.append(execution.input_file(mask))
    cargs.extend([execution.input_file(f) for f in input_files])
    ret = V3dtoXdatasetOutputs(
        root=execution.output_file("."),
        output_sdat=execution.output_file(prefix + ".sdat"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dtoXdatasetOutputs",
    "V_3DTO_XDATASET_METADATA",
    "v_3dto_xdataset",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

REGISTRATION_METADATA = Metadata(
    id="5f520d7384383baaf15d7e4094d989410172644f.boutiques",
    name="registration",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


@dataclasses.dataclass
class RegistrationInitialMovingTransform:
    initial_moving_transform: InputPathType
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append(execution.input_file(self.initial_moving_transform))
        return cargs


@dataclasses.dataclass
class RegistrationInitialMovingTransformUseInverse:
    initial_moving_transform: InputPathType
    use_inverse: typing.Literal[0, 1] | None = None
    """Use the inverse of the initial moving transform."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        if self.use_inverse is not None:
            cargs.append("[" + execution.input_file(self.initial_moving_transform) + "," + str(self.use_inverse) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationInitialMovingTransformInitializationFeature:
    fixed_image: InputPathType
    moving_image: InputPathType
    initialization_feature: typing.Literal[0, 1, 2]
    """Initialization feature. 0: Geometric center of images, 1: Image
    intensities, 2: Origin of images"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("[" + execution.input_file(self.fixed_image) + "," + execution.input_file(self.moving_image) + "," + str(self.initialization_feature) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationTransformRigid:
    gradient_step: float
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("Rigid[" + str(self.gradient_step) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationTransformAffine:
    gradient_step: float
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("Affine[" + str(self.gradient_step) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationTransformCompositeAffine:
    gradient_step: float
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("CompositeAffine[" + str(self.gradient_step) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationTransformSimilarity:
    gradient_step: float
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("Similarity[" + str(self.gradient_step) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationTransformTranslation:
    gradient_step: float
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("Translation[" + str(self.gradient_step) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationTransformBspline:
    gradient_step: float
    mesh_size_at_base_level: float
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("BSpline[" + str(self.gradient_step) + "," + str(self.mesh_size_at_base_level) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationTransformGaussianDisplacementField:
    gradient_step: float
    update_field_variance_in_voxel_space: float
    total_field_variance_in_voxel_space: float
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("GaussianDisplacementField[" + str(self.gradient_step) + "," + str(self.update_field_variance_in_voxel_space) + "," + str(self.total_field_variance_in_voxel_space) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationTransformBsplineDisplacementField:
    gradient_step: float
    update_field_mesh_size_at_base_level: float
    total_field_mesh_size_at_base_level: float | None = None
    spline_order: float | None = None
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        if self.total_field_mesh_size_at_base_level is not None and self.spline_order is not None:
            cargs.append("BSplineDisplacementField[" + str(self.gradient_step) + "," + str(self.update_field_mesh_size_at_base_level) + "," + str(self.total_field_mesh_size_at_base_level) + str(self.spline_order) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationTransformTimeVaryingVelocityField:
    gradient_step: float
    number_of_time_indices: float
    update_field_variance_in_voxel_space: float
    update_field_time_variance: float
    total_field_variance_in_voxel_space: float
    total_field_time_variance: float
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("TimeVaryingVelocityField[" + str(self.gradient_step) + "," + str(self.number_of_time_indices) + "," + str(self.update_field_variance_in_voxel_space) + "," + str(self.update_field_time_variance) + "," + str(self.total_field_variance_in_voxel_space) + "," + str(self.total_field_time_variance) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationTransformTimeVaryingBsplineVelocityField:
    gradient_step: float
    velocity_field_mesh_size: float
    number_of_time_point_samples: float | None = None
    spline_order: float | None = None
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        if self.number_of_time_point_samples is not None and self.spline_order is not None:
            cargs.append("TimeVaryingBSplineVelocityField[" + str(self.gradient_step) + "," + str(self.velocity_field_mesh_size) + "," + str(self.number_of_time_point_samples) + "," + str(self.spline_order) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationTransformSyn:
    gradient_step: float
    update_field_variance_in_voxel_space: float
    total_field_variance_in_voxel_space: float
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("SyN[" + str(self.gradient_step) + "," + str(self.update_field_variance_in_voxel_space) + "," + str(self.total_field_variance_in_voxel_space) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationTransformBsplineSyn:
    gradient_step: float
    update_field_mesh_size_at_base_level: float
    total_field_mesh_size_at_base_level: float | None = None
    spline_order: float | None = None
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        if self.total_field_mesh_size_at_base_level is not None and self.spline_order is not None:
            cargs.append("BSplineSyN[" + str(self.gradient_step) + "," + str(self.update_field_mesh_size_at_base_level) + "," + str(self.total_field_mesh_size_at_base_level) + "," + str(self.spline_order) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationTransformExponential:
    gradient_step: float
    update_field_variance_in_voxel_space: float
    velocity_field_variance_in_voxel_space: float
    number_of_integration_steps: float
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("Exponential[" + str(self.gradient_step) + "," + str(self.update_field_variance_in_voxel_space) + "," + str(self.velocity_field_variance_in_voxel_space) + "," + str(self.number_of_integration_steps) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationTransformBsplineExponential:
    gradient_step: float
    update_field_mesh_size_at_base_level: float
    velocity_field_mesh_size_at_base_level: float | None = None
    number_of_integration_steps: float | None = None
    spline_order: float | None = None
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        if self.velocity_field_mesh_size_at_base_level is not None and self.number_of_integration_steps is not None and self.spline_order is not None:
            cargs.append("BSplineExponential[" + str(self.gradient_step) + "," + str(self.update_field_mesh_size_at_base_level) + "," + str(self.velocity_field_mesh_size_at_base_level) + "," + str(self.number_of_integration_steps) + "," + str(self.spline_order) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationMetricAntsNeighbourhoodCrossCorrelation:
    fixed_image: str
    moving_image: str
    metric_weight: float
    radius: float | None = None
    sampling_strategy: typing.Literal["None", "Regular", "Random"] | None = None
    sampling_percentage: float | None = None
    use_gradient_filter: typing.Literal["true", "false"] | None = None
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        if sampling_percentage is not None and not (0 <= sampling_percentage <= 1): 
            raise ValueError(f"'sampling_percentage' must be between 0 <= x <= 1 but was {sampling_percentage}")
        cargs = []
        if self.radius is not None and self.sampling_strategy is not None and self.sampling_percentage is not None and self.use_gradient_filter is not None:
            cargs.append("CC[" + self.fixed_image + "," + self.moving_image + "," + str(self.metric_weight) + "," + str(self.radius) + "," + self.sampling_strategy + "," + str(self.sampling_percentage) + "," + self.use_gradient_filter + "]")
        return cargs


@dataclasses.dataclass
class RegistrationMetricMutualInformation:
    fixed_image: str
    moving_image: str
    metric_weight: float
    number_of_bins: float | None = None
    sampling_strategy: typing.Literal["None", "Regular", "Random"] | None = None
    sampling_percentage: float | None = None
    use_gradient_filter: typing.Literal["true", "false"] | None = None
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        if sampling_percentage is not None and not (0 <= sampling_percentage <= 1): 
            raise ValueError(f"'sampling_percentage' must be between 0 <= x <= 1 but was {sampling_percentage}")
        cargs = []
        if self.number_of_bins is not None and self.sampling_strategy is not None and self.sampling_percentage is not None and self.use_gradient_filter is not None:
            cargs.append("MI[" + self.fixed_image + "," + self.moving_image + "," + str(self.metric_weight) + "," + str(self.number_of_bins) + "," + self.sampling_strategy + "," + str(self.sampling_percentage) + "," + self.use_gradient_filter + "]")
        return cargs


@dataclasses.dataclass
class RegistrationMetricMattes:
    fixed_image: str
    moving_image: str
    metric_weight: float
    number_of_bins: float | None = None
    sampling_strategy: typing.Literal["None", "Regular", "Random"] | None = None
    sampling_percentage: float | None = None
    use_gradient_filter: typing.Literal["true", "false"] | None = None
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        if sampling_percentage is not None and not (0 <= sampling_percentage <= 1): 
            raise ValueError(f"'sampling_percentage' must be between 0 <= x <= 1 but was {sampling_percentage}")
        cargs = []
        if self.number_of_bins is not None and self.sampling_strategy is not None and self.sampling_percentage is not None and self.use_gradient_filter is not None:
            cargs.append("Mattes[" + self.fixed_image + "," + self.moving_image + "," + str(self.metric_weight) + "," + str(self.number_of_bins) + "," + self.sampling_strategy + "," + str(self.sampling_percentage) + "," + self.use_gradient_filter + "]")
        return cargs


@dataclasses.dataclass
class RegistrationMetricMeanSquares:
    fixed_image: str
    moving_image: str
    metric_weight: float
    radius: float | None = None
    sampling_strategy: typing.Literal["None", "Regular", "Random"] | None = None
    sampling_percentage: float | None = None
    use_gradient_filter: typing.Literal["true", "false"] | None = None
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        if sampling_percentage is not None and not (0 <= sampling_percentage <= 1): 
            raise ValueError(f"'sampling_percentage' must be between 0 <= x <= 1 but was {sampling_percentage}")
        cargs = []
        if self.radius is not None and self.sampling_strategy is not None and self.sampling_percentage is not None and self.use_gradient_filter is not None:
            cargs.append("MeanSquares[" + self.fixed_image + "," + self.moving_image + "," + str(self.metric_weight) + "," + str(self.radius) + "," + self.sampling_strategy + "," + str(self.sampling_percentage) + "," + self.use_gradient_filter + "]")
        return cargs


@dataclasses.dataclass
class RegistrationMetricDemons:
    fixed_image: str
    moving_image: str
    metric_weight: float
    number_of_bins: float | None = None
    sampling_strategy: typing.Literal["None", "Regular", "Random"] | None = None
    sampling_percentage: float | None = None
    use_gradient_filter: typing.Literal["true", "false"] | None = None
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        if sampling_percentage is not None and not (0 <= sampling_percentage <= 1): 
            raise ValueError(f"'sampling_percentage' must be between 0 <= x <= 1 but was {sampling_percentage}")
        cargs = []
        if self.number_of_bins is not None and self.sampling_strategy is not None and self.sampling_percentage is not None and self.use_gradient_filter is not None:
            cargs.append("Demons[" + self.fixed_image + "," + self.moving_image + "," + str(self.metric_weight) + "," + str(self.number_of_bins) + "," + self.sampling_strategy + "," + str(self.sampling_percentage) + "," + self.use_gradient_filter + "]")
        return cargs


@dataclasses.dataclass
class RegistrationMetricGlobalCorrelation:
    fixed_image: str
    moving_image: str
    metric_weight: float
    radius: float | None = None
    sampling_strategy: typing.Literal["None", "Regular", "Random"] | None = None
    sampling_percentage: float | None = None
    use_gradient_filter: typing.Literal["true", "false"] | None = None
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        if sampling_percentage is not None and not (0 <= sampling_percentage <= 1): 
            raise ValueError(f"'sampling_percentage' must be between 0 <= x <= 1 but was {sampling_percentage}")
        cargs = []
        if self.radius is not None and self.sampling_strategy is not None and self.sampling_percentage is not None and self.use_gradient_filter is not None:
            cargs.append("GC[" + self.fixed_image + "," + self.moving_image + "," + str(self.metric_weight) + "," + str(self.radius) + "," + self.sampling_strategy + "," + str(self.sampling_percentage) + "," + self.use_gradient_filter + "]")
        return cargs


@dataclasses.dataclass
class RegistrationMetricEuclideanIcp:
    fixed_point_set: str
    moving_point_set: str
    metric_weight: float
    sampling_percentage: float | None = None
    boundary_points_only: typing.Literal["0"] | None = None
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        if sampling_percentage is not None and not (0 <= sampling_percentage <= 1): 
            raise ValueError(f"'sampling_percentage' must be between 0 <= x <= 1 but was {sampling_percentage}")
        cargs = []
        if self.sampling_percentage is not None and self.boundary_points_only is not None:
            cargs.append("ICP[" + self.fixed_point_set + "," + self.moving_point_set + "," + str(self.metric_weight) + "," + str(self.sampling_percentage) + "," + self.boundary_points_only + "]")
        return cargs


@dataclasses.dataclass
class RegistrationMetricPointSetExpectation:
    metric_weight: float
    point_set_sigma: float | None = None
    sampling_percentage: float | None = None
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        if sampling_percentage is not None and not (0 <= sampling_percentage <= 1): 
            raise ValueError(f"'sampling_percentage' must be between 0 <= x <= 1 but was {sampling_percentage}")
        cargs = []
        if self.point_set_sigma is not None and self.sampling_percentage is not None:
            cargs.append("PSE[[FIXED_IMAGE],[MOVING_IMAGE]," + str(self.metric_weight) + "," + "," + str(self.point_set_sigma) + "[SAMPLING_STRATEGY]" + "," + str(self.sampling_percentage) + "[USE_GRADIENT_FILTER]]")
        return cargs


@dataclasses.dataclass
class RegistrationMetricJensenHavrdaCharvetTsallis:
    fixed_point_set: str
    moving_point_set: str
    metric_weight: float
    sampling_percentage: float | None = None
    boundary_points_only: typing.Literal["0"] | None = None
    point_set_sigma: float | None = None
    k_neighborhood: float | None = None
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        if sampling_percentage is not None and not (0 <= sampling_percentage <= 1): 
            raise ValueError(f"'sampling_percentage' must be between 0 <= x <= 1 but was {sampling_percentage}")
        cargs = []
        if self.sampling_percentage is not None and self.boundary_points_only is not None and self.point_set_sigma is not None and self.k_neighborhood is not None:
            cargs.append("JHCT[" + self.fixed_point_set + "," + self.moving_point_set + "," + str(self.metric_weight) + "," + str(self.sampling_percentage) + "," + self.boundary_points_only + "," + str(self.point_set_sigma) + "," + str(self.k_neighborhood) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationMetricIgdm:
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("IGDM[")
        return cargs


@dataclasses.dataclass
class RegistrationConvergence:
    """
    Convergence is determined from the number of iterations per level and is
    determined by fitting a line to the normalized energy profile of the last N
    iterations (where N is specified by the window size) and determining the
    slope which is then compared with the convergence threshold. .
    """
    convergence: str
    convergence_threshold: float
    convergence_window_size: int
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("[" + self.convergence + "," + str(self.convergence_threshold) + "," + str(self.convergence_window_size) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationStage:
    """
    Stages of the registration process.
    """
    transform: typing.Union[RegistrationTransformRigid, RegistrationTransformAffine, RegistrationTransformCompositeAffine, RegistrationTransformSimilarity, RegistrationTransformTranslation, RegistrationTransformBspline, RegistrationTransformGaussianDisplacementField, RegistrationTransformBsplineDisplacementField, RegistrationTransformTimeVaryingVelocityField, RegistrationTransformTimeVaryingBsplineVelocityField, RegistrationTransformSyn, RegistrationTransformBsplineSyn, RegistrationTransformExponential, RegistrationTransformBsplineExponential]
    """Several transform options are available. The gradientStep or learningRate
    characterizes the gradient descent optimization and is scaled appropriately
    for each transform using the shift scales estimator. Subsequent parameters
    are transform-specific and can be determined from the usage. For the
    B-spline transforms one can also specify the smoothing in terms of spline
    distance (i.e. knot spacing)."""
    metric: typing.Union[RegistrationMetricAntsNeighbourhoodCrossCorrelation, RegistrationMetricMutualInformation, RegistrationMetricMattes, RegistrationMetricMeanSquares, RegistrationMetricDemons, RegistrationMetricGlobalCorrelation, RegistrationMetricEuclideanIcp, RegistrationMetricPointSetExpectation, RegistrationMetricJensenHavrdaCharvetTsallis, RegistrationMetricIgdm]
    """These image metrics are available--- CC: ANTS neighborhood cross
    correlation, MI: Mutual information, Demons: (Thirion), MeanSquares, and GC:
    Global Correlation. The "metricWeight" variable is used to modulate the per
    stage weighting of the metrics. The metrics can also employ a sampling
    strategy defined by a sampling percentage. The sampling strategy defaults to
    'None' (aka a dense sampling of one sample per voxel), otherwise it defines
    a point set over which to optimize the metric. The point set can be on a
    regular lattice or a random lattice of points slightly perturbed to minimize
    aliasing artifacts. samplingPercentage defines the fraction of points to
    select from the domain. useGradientFilter specifies whether a
    smoothingfilter is applied when estimating the metric gradient.In addition,
    three point set metrics are available: Euclidean (ICP), Point-set
    expectation (PSE), and Jensen-Havrda-Charvet-Tsallis (JHCT)."""
    convergence: RegistrationConvergence
    """Convergence is determined from the number of iterations per level and is
    determined by fitting a line to the normalized energy profile of the last N
    iterations (where N is specified by the window size) and determining the
    slope which is then compared with the convergence threshold. """
    smoothing_sigmas: str
    """Specify the sigma of gaussian smoothing at each level. Units are given in
    terms of voxels ('vox') or physical spacing ('mm'). Example usage is
    '4x2x1mm' and '4x2x1vox' where no units implies voxel spacing."""
    shrink_factors: str
    """Specify the shrink factor for the virtual domain (typically the fixed
    image) at each level."""
    use_histogram_matching: typing.Literal[0, 1] | None = None
    """Use histogram matching."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.extend([
            "--transform",
            *self.transform.run(execution)
        ])
        cargs.extend([
            "--metric",
            *self.metric.run(execution)
        ])
        cargs.extend([
            "--convergence",
            *self.convergence.run(execution)
        ])
        cargs.extend([
            "--smoothing-sigmas",
            self.smoothing_sigmas
        ])
        cargs.extend([
            "--shrink-factors",
            self.shrink_factors
        ])
        if self.use_histogram_matching is not None:
            cargs.extend([
                "--use-histogram-matching",
                str(self.use_histogram_matching)
            ])
        return cargs


@dataclasses.dataclass
class RegistrationWinsorizeImageIntensities:
    """
    Winsorize data based on specified quantiles.
    """
    lower_quantile: float
    upper_quantile: float
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("[" + str(self.lower_quantile) + str(self.upper_quantile) + "]")
        return cargs


@dataclasses.dataclass
class RegistrationMasks:
    """
    Image masks to limit voxels considered by the metric. Two options are
    allowed for mask specification: 1) Either the user specifies a single mask
    to be used for all stages or 2) the user specifies a mask for each stage.
    With the latter one can select to which stages masks are applied by
    supplying valid file names. If the file does not exist, a mask will not be
    used for that stage. Note that we handle the fixed and moving masks
    separately to enforce this constraint.
    """
    fixed_mask: str | None = None
    moving_mask: str | None = None
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        if self.fixed_mask is not None and self.moving_mask is not None:
            cargs.append("[" + self.fixed_mask + self.moving_mask + "]")
        return cargs


class RegistrationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `registration(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    generic_affine: OutputPathType | None
    """The output is the generic affine transformation matrix."""
    inverse_warped: OutputPathType | None
    """The output is the warped fixed image."""
    inverse_warp: OutputPathType | None
    """The output is the inverse warp field."""
    warped: OutputPathType | None
    """The output is the warped moving image."""
    warp: OutputPathType | None
    """The output is the warp field."""


def registration(
    stages: list[RegistrationStage],
    dimensionality: typing.Literal[2, 3, 4] | None = None,
    output: str | None = None,
    save_state: str | None = None,
    restore_state: str | None = None,
    write_composite_transform: typing.Literal[0, 1] | None = None,
    print_similarity_measure_interval: int | None = None,
    write_interval_volumes: int | None = None,
    collapse_output_transforms: typing.Literal[1, 0] | None = None,
    initialize_transforms_per_stage: typing.Literal[1, 0] | None = None,
    interpolation: typing.Literal["Linear", "NearestNeighbor", "MultiLabel", "Gaussian", "BSpline", "CosineWindowedSinc", "WelchWindowedSinc", "HammingWindowedSinc", "LanczosWindowedSinc", "GenericLabel"] | None = None,
    restrict_deformation: list[typing.Literal[0, 1]] | None = None,
    initial_fixed_transform: str | None = None,
    initial_moving_transform: typing.Union[RegistrationInitialMovingTransform, RegistrationInitialMovingTransformUseInverse, RegistrationInitialMovingTransformInitializationFeature] | None = None,
    winsorize_image_intensities: RegistrationWinsorizeImageIntensities | None = None,
    masks: RegistrationMasks | None = None,
    minc: typing.Literal[0, 1] | None = None,
    random_seed: int | None = None,
    verbose: typing.Literal[0, 1] | None = None,
    runner: Runner | None = None,
) -> RegistrationOutputs:
    """
    This program is a user-level registration application meant to utilize classes
    in ITK v4.0 and later. The user can specify any number of "stages" where a stage
    consists of a transform; an image metric; and iterations, shrink factors, and
    smoothing sigmas for each level. Note that explicitly setting the
    dimensionality, metric, transform, output, convergence, shrink-factors, and
    smoothing-sigmas parameters is mandatory.
    
    Author: Advanced Normalization Tools (ANTs) Contributors
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        stages: Stages of the registration process.
        dimensionality: This option forces the image to be treated as a\
            specified-dimensional image. If not specified, we try to infer the\
            dimensionality from the input image.
        output: Specify the output transform prefix (output format is .nii.gz\
            ). Optionally, one can choose to warp the moving image to the fixed\
            space and, if the inverse transform exists, one can also output the\
            warped fixed image. Note that only the images specified in the first\
            metric call are warped. Use antsApplyTransforms to warp other images\
            using the resultant transform(s). When a composite transform is not\
            specified, linear transforms are specified with a '.mat' suffix and\
            displacement fields with a 'Warp.nii.gz' suffix (and\
            'InverseWarp.nii.gz', when applicable. In addition, for velocity-based\
            transforms, the full velocity field is written to file\
            ('VelocityField.nii.gz') as long as the collapse transforms flag is\
            turned off ('-z 0').
        save_state: Specify the output file for the current state of the\
            registration. The state file is written to an hdf5 composite file. It\
            is specially usefull if we want to save the current state of a SyN\
            registration to the disk, so we can load and restore that later to\
            continue the next registration process directly started from the last\
            saved state. The output file of this flag is the same as the\
            write-composite-transform, unless the last transform is a SyN\
            transform. In that case, the inverse displacement field of the SyN\
            transform is also added to the output composite transform. Again notice\
            that this file cannot be treated as a transform, and restore-state\
            option must be used to load the written file by this flag.
        restore_state: Specify the initial state of the registration which get\
            immediately used to directly initialize the registration process. The\
            flag is mutually exclusive with other intialization flags.If this flag\
            is used, none of the initial-moving-transform and\
            initial-fixed-transform cannot be used.
        write_composite_transform: Boolean specifying whether or not the\
            composite transform (and its inverse, if it exists) should be written\
            to an hdf5 composite file. This is false by default so that only the\
            transform for each stage is written to file.
        print_similarity_measure_interval: Prints out the CC similarity metric\
            measure between the full-size input fixed and the transformed moving\
            images at each iteration a value of 0 (the default) indicates that the\
            full scale computation should not take placeany value greater than 0\
            represents the interval of full scale metric computation.
        write_interval_volumes: Writes out the output volume at each iteration.\
            It helps to present the registration process as a short movie a value\
            of 0 (the default) indicates that this option should not take placeany\
            value greater than 0 represents the interval between the iterations\
            which outputs are written to the disk.
        collapse_output_transforms: Collapse output transforms. Specifically,\
            enabling this option combines all adjacent transforms where possible.\
            All adjacent linear transforms are written to disk in the form of an\
            itk affine transform (called xxxGenericAffine.mat).\
            Similarly, all adjacent displacement field transforms are combined\
            when written to disk (e.g. xxxWarp.nii.gz and xxxInverseWarp.nii.gz\
            (if available)). Also, an output composite transform including the\
            collapsed transforms is written to the disk (called\
            outputCollapsed(Inverse)Composite).
        initialize_transforms_per_stage: Initialize linear transforms from the\
            previous stage. By enabling this option, the current linear stage\
            transform is directly intialized from the previous stage's linear\
            transform; this allows multiple linear stages to be run where each\
            stage directly updates the estimated linear transform from the previous\
            stage. (e.g. Translation -> Rigid -> Affine).
        interpolation: Several interpolation options are available in ITK.\
            These have all been made available. Currently the interpolator choice\
            is only used to warp (and possibly inverse warp) the final output\
            image(s).
        restrict_deformation: This option allows the user to restrict the\
            optimization of the displacement field, translation, rigid or affine\
            transform on a per-component basis. For example, if one wants to limit\
            the deformation or rotation of 3-D volume to the first two dimensions,\
            this is possible by specifying a weight vector of '1x1x0' for a\
            deformation field or '1x1x0x1x1x0' for a rigid transformation.\
            Low-dimensional restriction only works if there are no preceding\
            transformations.All stages up to and including the desired stage must\
            have this option specified,even if they should not be restricted (in\
            which case specify 1x1x1...).
        initial_fixed_transform: Specify the initial fixed transform(s) which\
            get immediately incorporated into the composite transform. The order of\
            the transforms is stack-esque in that the last transform specified on\
            the command line is the first to be applied. In addition to\
            initialization with ITK transforms, the user can perform an initial\
            translation alignment by specifying the fixed and moving images and\
            selecting an initialization feature. These features include using the\
            geometric center of the images (=0), the image intensities (=1), or the\
            origin of the images (=2).
        initial_moving_transform: Specify the initial moving transform(s) which\
            get immediately incorporated into the composite transform. The order of\
            the transforms is stack-esque in that the last transform specified on\
            the command line is the first to be applied. In addition to\
            initialization with ITK transforms, the user can perform an initial\
            translation alignment by specifying the fixed and moving images and\
            selecting an initialization feature. These features include using the\
            geometric center of the images (=0), the image intensities (=1), or the\
            origin of the images (=2).
        winsorize_image_intensities: Winsorize data based on specified\
            quantiles.
        masks: Image masks to limit voxels considered by the metric. Two\
            options are allowed for mask specification: 1) Either the user\
            specifies a single mask to be used for all stages or 2) the user\
            specifies a mask for each stage. With the latter one can select to\
            which stages masks are applied by supplying valid file names. If the\
            file does not exist, a mask will not be used for that stage. Note that\
            we handle the fixed and moving masks separately to enforce this\
            constraint.
        minc: Use MINC file formats for transformations.
        random_seed: Random seed.
        verbose: Verbose output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `RegistrationOutputs`).
    """
    if write_interval_volumes is not None and not (0 <= write_interval_volumes): 
        raise ValueError(f"'write_interval_volumes' must be greater than 0 <= x but was {write_interval_volumes}")
    if random_seed is not None and not (1 <= random_seed): 
        raise ValueError(f"'random_seed' must be greater than 1 <= x but was {random_seed}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(REGISTRATION_METADATA)
    cargs = []
    cargs.append("antsRegistration")
    if dimensionality is not None:
        cargs.extend([
            "--dimensionality",
            str(dimensionality)
        ])
    if output is not None:
        cargs.extend([
            "-o",
            output
        ])
    if save_state is not None:
        cargs.extend([
            "-j",
            save_state
        ])
    if restore_state is not None:
        cargs.extend([
            "-k",
            restore_state
        ])
    if write_composite_transform is not None:
        cargs.extend([
            "-a",
            str(write_composite_transform)
        ])
    if print_similarity_measure_interval is not None:
        cargs.extend([
            "-p",
            str(print_similarity_measure_interval)
        ])
    if write_interval_volumes is not None:
        cargs.extend([
            "--write-interval-volumes",
            str(write_interval_volumes)
        ])
    if collapse_output_transforms is not None:
        cargs.extend([
            "--collapse-output-transforms",
            str(collapse_output_transforms)
        ])
    if initialize_transforms_per_stage is not None:
        cargs.extend([
            "-i",
            str(initialize_transforms_per_stage)
        ])
    if interpolation is not None:
        cargs.extend([
            "--interpolation",
            interpolation
        ])
    if restrict_deformation is not None:
        cargs.extend([
            "-g",
            "x".join(map(str, restrict_deformation))
        ])
    if initial_fixed_transform is not None:
        cargs.extend([
            "-q",
            initial_fixed_transform
        ])
    if initial_moving_transform is not None:
        cargs.extend([
            "--initial-moving-transform",
            *initial_moving_transform.run(execution)
        ])
    cargs.extend([a for c in [s.run(execution) for s in stages] for a in c])
    if winsorize_image_intensities is not None:
        cargs.extend([
            "--winsorize-image-intensities",
            *winsorize_image_intensities.run(execution)
        ])
    if masks is not None:
        cargs.extend([
            "--masks",
            *masks.run(execution)
        ])
    if minc is not None:
        cargs.extend([
            "--minc",
            str(minc)
        ])
    if random_seed is not None:
        cargs.extend([
            "--random-seed",
            str(random_seed)
        ])
    if verbose is not None:
        cargs.extend([
            "-v",
            str(verbose)
        ])
    ret = RegistrationOutputs(
        root=execution.output_file("."),
        generic_affine=execution.output_file(output + "0GenericAffine.mat") if (output is not None) else None,
        inverse_warped=execution.output_file(output + "InverseWarped.nii.gz") if (output is not None) else None,
        inverse_warp=execution.output_file(output + "1InverseWarp.nii.gz") if (output is not None) else None,
        warped=execution.output_file(output + "Warped.nii.gz") if (output is not None) else None,
        warp=execution.output_file(output + "1Warp.nii.gz") if (output is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "REGISTRATION_METADATA",
    "RegistrationConvergence",
    "RegistrationInitialMovingTransform",
    "RegistrationInitialMovingTransformInitializationFeature",
    "RegistrationInitialMovingTransformUseInverse",
    "RegistrationMasks",
    "RegistrationMetricAntsNeighbourhoodCrossCorrelation",
    "RegistrationMetricDemons",
    "RegistrationMetricEuclideanIcp",
    "RegistrationMetricGlobalCorrelation",
    "RegistrationMetricIgdm",
    "RegistrationMetricJensenHavrdaCharvetTsallis",
    "RegistrationMetricMattes",
    "RegistrationMetricMeanSquares",
    "RegistrationMetricMutualInformation",
    "RegistrationMetricPointSetExpectation",
    "RegistrationOutputs",
    "RegistrationStage",
    "RegistrationTransformAffine",
    "RegistrationTransformBspline",
    "RegistrationTransformBsplineDisplacementField",
    "RegistrationTransformBsplineExponential",
    "RegistrationTransformBsplineSyn",
    "RegistrationTransformCompositeAffine",
    "RegistrationTransformExponential",
    "RegistrationTransformGaussianDisplacementField",
    "RegistrationTransformRigid",
    "RegistrationTransformSimilarity",
    "RegistrationTransformSyn",
    "RegistrationTransformTimeVaryingBsplineVelocityField",
    "RegistrationTransformTimeVaryingVelocityField",
    "RegistrationTransformTranslation",
    "RegistrationWinsorizeImageIntensities",
    "registration",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__ISO_MASKS_METADATA = Metadata(
    id="5e08bfc1c4611e9acd94b8f91b895e983973e1ef.boutiques",
    name="@IsoMasks",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VIsoMasksOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__iso_masks(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def v__iso_masks(
    input_dataset: InputPathType,
    isovals: list[float] | None = None,
    runner: Runner | None = None,
) -> VIsoMasksOutputs:
    """
    Creates isosurfaces from isovolume envelopes.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/@IsoMasks.html
    
    Args:
        input_dataset: Input dataset for creating isosurfaces.
        isovals: Isovalue thresholds for creating isosurfaces.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VIsoMasksOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__ISO_MASKS_METADATA)
    cargs = []
    cargs.append("@IsoMasks")
    cargs.append("-mask")
    cargs.extend([
        "-mask",
        execution.input_file(input_dataset)
    ])
    if isovals is not None:
        cargs.extend(map(str, isovals))
    ret = VIsoMasksOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VIsoMasksOutputs",
    "V__ISO_MASKS_METADATA",
    "v__iso_masks",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FAT_PROC_SELECT_VOLS_METADATA = Metadata(
    id="222d5c71dc9e688c8a3e2410fa4b1dc5b5b84158.boutiques",
    name="fat_proc_select_vols",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class FatProcSelectVolsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fat_proc_select_vols(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_selector_string: OutputPathType
    """Text file with AFNI-usable selector string"""


def fat_proc_select_vols(
    dwi_input: InputPathType,
    img_input: InputPathType,
    prefix: str,
    in_bads: InputPathType | None = None,
    apply_to_vols: bool = False,
    do_movie: str | None = None,
    workdir: str | None = None,
    no_cmd_out: bool = False,
    runner: Runner | None = None,
) -> FatProcSelectVolsOutputs:
    """
    Tool for building a selector string for AFNI subbricks and/or 1D text files.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/fat_proc_select_vols.html
    
    Args:
        dwi_input: Input DWI dataset.
        img_input: 2D image of the DWI dataset.
        prefix: Output prefix for files.
        in_bads: A single column file of integers representing bad volumes\
            indices (optional).
        apply_to_vols: Apply the created selection of good volumes to the DWI\
            dataset.
        do_movie: Output a movie of the newly created dataset. Only 'AGIF' or\
            'MPEG' arguments can be used.
        workdir: Specify a working directory.
        no_cmd_out: Don't save the command line call of this program and the\
            location where it was run.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FatProcSelectVolsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FAT_PROC_SELECT_VOLS_METADATA)
    cargs = []
    cargs.append("fat_proc_select_vols")
    cargs.append("-in_dwi")
    cargs.append(execution.input_file(dwi_input))
    cargs.append("-in_img")
    cargs.append(execution.input_file(img_input))
    cargs.append("-prefix")
    cargs.append(prefix)
    if in_bads is not None:
        cargs.extend([
            "-in_bads",
            execution.input_file(in_bads)
        ])
    if apply_to_vols:
        cargs.append("-apply_to_vols")
    if do_movie is not None:
        cargs.extend([
            "-do_movie",
            do_movie
        ])
    if workdir is not None:
        cargs.extend([
            "-workdir",
            workdir
        ])
    if no_cmd_out:
        cargs.append("-no_cmd_out")
    ret = FatProcSelectVolsOutputs(
        root=execution.output_file("."),
        output_selector_string=execution.output_file(prefix + "_bads.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FAT_PROC_SELECT_VOLS_METADATA",
    "FatProcSelectVolsOutputs",
    "fat_proc_select_vols",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_BALL_MATCH_METADATA = Metadata(
    id="ec86e9d0ff8252e6076f6e6033b6c38607b33cd2.boutiques",
    name="3dBallMatch",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dBallMatchOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_ball_match(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_stdout: OutputPathType
    """Output containing matching coordinates and related data"""


def v_3d_ball_match(
    input_dataset: InputPathType,
    radius: float | None = None,
    runner: Runner | None = None,
) -> V3dBallMatchOutputs:
    """
    A tool to find a good match between a ball (filled sphere) of the given radius
    and a dataset to determine a crude approximate center of the brain quickly.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dBallMatch.html
    
    Args:
        input_dataset: Input dataset (e.g., Fred.nii).
        radius: Radius of the 3D ball to match (in mm).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dBallMatchOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_BALL_MATCH_METADATA)
    cargs = []
    cargs.append("3dBallMatch")
    cargs.append("[OPTIONS]")
    cargs.append(execution.input_file(input_dataset))
    if radius is not None:
        cargs.append(str(radius))
    ret = V3dBallMatchOutputs(
        root=execution.output_file("."),
        output_stdout=execution.output_file("stdout"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dBallMatchOutputs",
    "V_3D_BALL_MATCH_METADATA",
    "v_3d_ball_match",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

METRIC_ESTIMATE_FWHM_METADATA = Metadata(
    id="742556a08415c28c3256b8b5211382b5fcb3d725.boutiques",
    name="metric-estimate-fwhm",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


class MetricEstimateFwhmOutputs(typing.NamedTuple):
    """
    Output object returned when calling `metric_estimate_fwhm(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def metric_estimate_fwhm(
    surface: InputPathType,
    metric_in: InputPathType,
    opt_roi_roi_metric: InputPathType | None = None,
    opt_column_column: str | None = None,
    opt_whole_file: bool = False,
    opt_demean: bool = False,
    runner: Runner | None = None,
) -> MetricEstimateFwhmOutputs:
    """
    Estimate fwhm smoothness of a metric file.
    
    Estimates the smoothness of the metric columns, printing the estimates to
    standard output. These estimates ignore variation in vertex spacing.
    
    Author: Washington University School of Medicin
    
    Args:
        surface: the surface to use for distance and neighbor information.
        metric_in: the input metric.
        opt_roi_roi_metric: use only data within an ROI: the metric file to use\
            as an ROI.
        opt_column_column: select a single column to estimate smoothness of:\
            the column number or name.
        opt_whole_file: estimate for the whole file at once, not each column\
            separately.
        opt_demean: subtract the mean image before estimating smoothness.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricEstimateFwhmOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_ESTIMATE_FWHM_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-metric-estimate-fwhm")
    cargs.append(execution.input_file(surface))
    cargs.append(execution.input_file(metric_in))
    if opt_roi_roi_metric is not None:
        cargs.extend([
            "-roi",
            execution.input_file(opt_roi_roi_metric)
        ])
    if opt_column_column is not None:
        cargs.extend([
            "-column",
            opt_column_column
        ])
    if opt_whole_file:
        cargs.append("-whole-file")
    if opt_demean:
        cargs.append("-demean")
    ret = MetricEstimateFwhmOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "METRIC_ESTIMATE_FWHM_METADATA",
    "MetricEstimateFwhmOutputs",
    "metric_estimate_fwhm",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FILE_CONVERT_METADATA = Metadata(
    id="a181c8fc965358072253814b8c8623961abda904.boutiques",
    name="file-convert",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class FileConvertBorderVersionConvert:
    """
    write a border file with a different version.
    """
    border_in: InputPathType
    """the input border file"""
    out_version: int
    """the format version to write as, 1 or 3 (2 doesn't exist)"""
    border_out: str
    """output - the output border file"""
    opt_surface_surface: InputPathType | None = None
    """must be specified if the input is version 1: use this surface file for
    structure and number of vertices, ignore borders on other structures"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-border-version-convert")
        cargs.append(execution.input_file(self.border_in))
        cargs.append(str(self.out_version))
        cargs.append(self.border_out)
        if self.opt_surface_surface is not None:
            cargs.extend([
                "-surface",
                execution.input_file(self.opt_surface_surface)
            ])
        return cargs


@dataclasses.dataclass
class FileConvertNiftiVersionConvert:
    """
    write a nifti file with a different version.
    """
    input_: str
    """the input nifti file"""
    version: int
    """the nifti version to write as"""
    output: str
    """output - the output nifti file"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-nifti-version-convert")
        cargs.append(self.input_)
        cargs.append(str(self.version))
        cargs.append(self.output)
        return cargs


@dataclasses.dataclass
class FileConvertCiftiVersionConvert:
    """
    write a cifti file with a different version.
    """
    cifti_in: InputPathType
    """the input cifti file"""
    version: str
    """the cifti version to write as"""
    cifti_out: str
    """output - the output cifti file"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-cifti-version-convert")
        cargs.append(execution.input_file(self.cifti_in))
        cargs.append(self.version)
        cargs.append(self.cifti_out)
        return cargs


class FileConvertOutputs(typing.NamedTuple):
    """
    Output object returned when calling `file_convert(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def file_convert(
    border_version_convert: FileConvertBorderVersionConvert | None = None,
    nifti_version_convert: FileConvertNiftiVersionConvert | None = None,
    cifti_version_convert: FileConvertCiftiVersionConvert | None = None,
    runner: Runner | None = None,
) -> FileConvertOutputs:
    """
    Change version of file format.
    
    You may only specify one top-level option.
    
    Author: Washington University School of Medicin
    
    Args:
        border_version_convert: write a border file with a different version.
        nifti_version_convert: write a nifti file with a different version.
        cifti_version_convert: write a cifti file with a different version.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FileConvertOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FILE_CONVERT_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-file-convert")
    if border_version_convert is not None:
        cargs.extend(border_version_convert.run(execution))
    if nifti_version_convert is not None:
        cargs.extend(nifti_version_convert.run(execution))
    if cifti_version_convert is not None:
        cargs.extend(cifti_version_convert.run(execution))
    ret = FileConvertOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FILE_CONVERT_METADATA",
    "FileConvertBorderVersionConvert",
    "FileConvertCiftiVersionConvert",
    "FileConvertNiftiVersionConvert",
    "FileConvertOutputs",
    "file_convert",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_VOL2_SURF_METADATA = Metadata(
    id="44242cf0a18b729b45c5d2950a88ab498b5b72e8.boutiques",
    name="3dVol2Surf",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dVol2SurfOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_vol2_surf(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_1d_file: OutputPathType | None
    """1D output file"""
    out_niml_file: OutputPathType | None
    """NIML output file"""
    seg_coords_file: OutputPathType | None
    """Segment coordinates output file"""


def v_3d_vol2_surf(
    spec_file: InputPathType,
    sv: InputPathType,
    grid_parent: InputPathType,
    map_func: str,
    surf_a: str,
    surf_b: str | None = None,
    out_1_d: str | None = None,
    out_niml: str | None = None,
    use_norms: bool = False,
    norm_len: float | None = None,
    first_node: float | None = None,
    last_node: float | None = None,
    debug_level: float | None = None,
    dnode: float | None = None,
    f_steps: float | None = None,
    f_index: str | None = None,
    f_p1_mm: float | None = None,
    f_pn_mm: float | None = None,
    f_p1_fr: float | None = None,
    f_pn_fr: float | None = None,
    skip_col_nodes: bool = False,
    skip_col_1dindex: bool = False,
    skip_col_i: bool = False,
    skip_col_j: bool = False,
    skip_col_k: bool = False,
    skip_col_vals: bool = False,
    no_headers: bool = False,
    save_seg_coords: str | None = None,
    cmask: str | None = None,
    gp_index: float | None = None,
    oob_index: float | None = None,
    oob_value: float | None = None,
    oom_value: float | None = None,
    outcols_afni_nsd: bool = False,
    outcols_1_result: bool = False,
    outcols_results: bool = False,
    outcols_nsd_format: bool = False,
    help_: bool = False,
    hist: bool = False,
    version: bool = False,
    keep_norm_dir: bool = False,
    reverse_norm_dir: bool = False,
    runner: Runner | None = None,
) -> V3dVol2SurfOutputs:
    """
    Map data from a volume domain to a surface domain.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dVol2Surf.html
    
    Args:
        spec_file: SUMA spec file.
        sv: AFNI volume dataset mapped by the surface.
        grid_parent: AFNI volume dataset used as a grid and orientation master\
            for output.
        map_func: Filter for values along the segment.
        surf_a: Name of surface A from the spec file.
        surf_b: Name of surface B from the spec file.
        out_1_d: Specify a 1D file for the output.
        out_niml: Specify a niml file for the output.
        use_norms: Use normals for second surface.
        norm_len: Length for node normals.
        first_node: Skip all previous nodes.
        last_node: Skip all following nodes.
        debug_level: Verbose output level.
        dnode: Node for debug.
        f_steps: Number of steps along each segment (defines the number of\
            evenly spaced points along each segment).
        f_index: Whether to use all segment point values or only those\
            corresponding to unique volume voxels.
        f_p1_mm: Distance in millimeters to add to the first point of each line\
            segment.
        f_pn_mm: Distance in millimeters to add to the second point of each\
            line segment.
        f_p1_fr: Fractional distance to add to the first point of each line\
            segment.
        f_pn_fr: Fractional distance to add to the second point of each line\
            segment.
        skip_col_nodes: Do not output node column.
        skip_col_1dindex: Do not output 1dindex column.
        skip_col_i: Do not output i column.
        skip_col_j: Do not output j column.
        skip_col_k: Do not output k column.
        skip_col_vals: Do not output vals column.
        no_headers: Do not output column headers.
        save_seg_coords: Save segment coordinates to a file.
        cmask: Command for dataset mask.
        gp_index: Choose grid_parent sub-brick.
        oob_index: Specify default index for out of bounds nodes.
        oob_value: Specify default value for out of bounds nodes.
        oom_value: Specify default value for out of mask nodes.
        outcols_afni_nsd: Output nodes and one result column.
        outcols_1_result: Output only one result column.
        outcols_results: Output only all result columns.
        outcols_nsd_format: Output nodes and all results (NI_SURF_DSET format).
        help_: Show this help.
        hist: Show revision history.
        version: Show version information.
        keep_norm_dir: Keep the direction of the normals.
        reverse_norm_dir: Reverse the normal directions.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dVol2SurfOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_VOL2_SURF_METADATA)
    cargs = []
    cargs.append("3dVol2Surf")
    cargs.append(execution.input_file(spec_file))
    cargs.extend([
        "-sv",
        execution.input_file(sv)
    ])
    cargs.extend([
        "-grid_parent",
        execution.input_file(grid_parent)
    ])
    cargs.extend([
        "-map_func",
        map_func
    ])
    cargs.extend([
        "-surf_A",
        surf_a
    ])
    if surf_b is not None:
        cargs.extend([
            "-surf_B",
            surf_b
        ])
    if out_1_d is not None:
        cargs.extend([
            "-out_1D",
            out_1_d
        ])
    if out_niml is not None:
        cargs.extend([
            "-out_niml",
            out_niml
        ])
    if use_norms:
        cargs.append("-use_norms")
    if norm_len is not None:
        cargs.extend([
            "-norm_len",
            str(norm_len)
        ])
    if first_node is not None:
        cargs.extend([
            "-first_node",
            str(first_node)
        ])
    if last_node is not None:
        cargs.extend([
            "-last_node",
            str(last_node)
        ])
    if debug_level is not None:
        cargs.extend([
            "-debug",
            str(debug_level)
        ])
    if dnode is not None:
        cargs.extend([
            "-dnode",
            str(dnode)
        ])
    if f_steps is not None:
        cargs.extend([
            "-f_steps",
            str(f_steps)
        ])
    if f_index is not None:
        cargs.extend([
            "-f_index",
            f_index
        ])
    if f_p1_mm is not None:
        cargs.extend([
            "-f_p1_mm",
            str(f_p1_mm)
        ])
    if f_pn_mm is not None:
        cargs.extend([
            "-f_pn_mm",
            str(f_pn_mm)
        ])
    if f_p1_fr is not None:
        cargs.extend([
            "-f_p1_fr",
            str(f_p1_fr)
        ])
    if f_pn_fr is not None:
        cargs.extend([
            "-f_pn_fr",
            str(f_pn_fr)
        ])
    if skip_col_nodes:
        cargs.append("-skip_col_nodes")
    if skip_col_1dindex:
        cargs.append("-skip_col_1dindex")
    if skip_col_i:
        cargs.append("-skip_col_i")
    if skip_col_j:
        cargs.append("-skip_col_j")
    if skip_col_k:
        cargs.append("-skip_col_k")
    if skip_col_vals:
        cargs.append("-skip_col_vals")
    if no_headers:
        cargs.append("-no_headers")
    if save_seg_coords is not None:
        cargs.extend([
            "-save_seg_coords",
            save_seg_coords
        ])
    if cmask is not None:
        cargs.extend([
            "-cmask",
            cmask
        ])
    if gp_index is not None:
        cargs.extend([
            "-gp_index",
            str(gp_index)
        ])
    if oob_index is not None:
        cargs.extend([
            "-oob_index",
            str(oob_index)
        ])
    if oob_value is not None:
        cargs.extend([
            "-oob_value",
            str(oob_value)
        ])
    if oom_value is not None:
        cargs.extend([
            "-oom_value",
            str(oom_value)
        ])
    if outcols_afni_nsd:
        cargs.append("-outcols_afni_NSD")
    if outcols_1_result:
        cargs.append("-outcols_1_result")
    if outcols_results:
        cargs.append("-outcols_results")
    if outcols_nsd_format:
        cargs.append("-outcols_NSD_format")
    if help_:
        cargs.append("-help")
    if hist:
        cargs.append("-hist")
    if version:
        cargs.append("-version")
    if keep_norm_dir:
        cargs.append("-keep_norm_dir")
    if reverse_norm_dir:
        cargs.append("-reverse_norm_dir")
    ret = V3dVol2SurfOutputs(
        root=execution.output_file("."),
        out_1d_file=execution.output_file(out_1_d) if (out_1_d is not None) else None,
        out_niml_file=execution.output_file(out_niml) if (out_niml is not None) else None,
        seg_coords_file=execution.output_file(save_seg_coords) if (save_seg_coords is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dVol2SurfOutputs",
    "V_3D_VOL2_SURF_METADATA",
    "v_3d_vol2_surf",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SURFACE_AVERAGE_METADATA = Metadata(
    id="a25b902d43dd15de7365d1cee60e024b418193a8.boutiques",
    name="surface-average",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class SurfaceAverageSurf:
    """
    specify a surface to include in the average.
    """
    surface: InputPathType
    """a surface file to average"""
    opt_weight_weight: float | None = None
    """specify a weighted average: the weight to use (default 1)"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-surf")
        cargs.append(execution.input_file(self.surface))
        if self.opt_weight_weight is not None:
            cargs.extend([
                "-weight",
                str(self.opt_weight_weight)
            ])
        return cargs


class SurfaceAverageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `surface_average(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    surface_out: OutputPathType
    """the output averaged surface"""
    stddev_metric_out: OutputPathType
    """the output metric for 3D sample standard deviation"""
    uncert_metric_out: OutputPathType
    """the output metric for uncertainty"""


def surface_average(
    surface_out: str,
    stddev_metric_out: str,
    uncert_metric_out: str,
    opt_stddev: bool = False,
    opt_uncertainty: bool = False,
    surf: list[SurfaceAverageSurf] | None = None,
    runner: Runner | None = None,
) -> SurfaceAverageOutputs:
    """
    Average surface files together.
    
    The 3D sample standard deviation is computed as 'sqrt(sum(squaredlength(xyz
    - mean(xyz)))/(n - 1))'.
    
    Uncertainty is a legacy measure used in caret5, and is computed as
    'sum(length(xyz - mean(xyz)))/n'.
    
    When weights are used, the 3D sample standard deviation treats them as
    reliability weights.
    
    Author: Washington University School of Medicin
    
    Args:
        surface_out: the output averaged surface.
        stddev_metric_out: the output metric for 3D sample standard deviation.
        uncert_metric_out: the output metric for uncertainty.
        opt_stddev: compute 3D sample standard deviation.
        opt_uncertainty: compute caret5 'uncertainty'.
        surf: specify a surface to include in the average.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceAverageOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_AVERAGE_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-surface-average")
    cargs.append(surface_out)
    if opt_stddev:
        cargs.append("-stddev")
    cargs.append(stddev_metric_out)
    if opt_uncertainty:
        cargs.append("-uncertainty")
    cargs.append(uncert_metric_out)
    if surf is not None:
        cargs.extend([a for c in [s.run(execution) for s in surf] for a in c])
    ret = SurfaceAverageOutputs(
        root=execution.output_file("."),
        surface_out=execution.output_file(surface_out),
        stddev_metric_out=execution.output_file(stddev_metric_out),
        uncert_metric_out=execution.output_file(uncert_metric_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SURFACE_AVERAGE_METADATA",
    "SurfaceAverageOutputs",
    "SurfaceAverageSurf",
    "surface_average",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_1D_SEM_METADATA = Metadata(
    id="7ece5c988136c06fa11babb83d522cb64a11016b.boutiques",
    name="1dSEM",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V1dSemOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_1d_sem(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Output printed to the terminal. This file may be redirected."""


def v_1d_sem(
    theta: InputPathType,
    correlation_matrix: InputPathType,
    residual_variance: InputPathType,
    degrees_of_freedom: float,
    runner: Runner | None = None,
) -> V1dSemOutputs:
    """
    Computes path coefficients for connection matrix in Structural Equation Modeling
    (SEM).
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/1dSEM.html
    
    Args:
        theta: Connection matrix 1D file with initial representation.
        correlation_matrix: Correlation matrix 1D file.
        residual_variance: Residual variance vector 1D file.
        degrees_of_freedom: Degrees of freedom.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V1dSemOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_1D_SEM_METADATA)
    cargs = []
    cargs.append("1dSEM")
    cargs.append("-theta")
    cargs.append(execution.input_file(theta))
    cargs.append("-C")
    cargs.append(execution.input_file(correlation_matrix))
    cargs.append("-psi")
    cargs.append(execution.input_file(residual_variance))
    cargs.append("-DF")
    cargs.append(str(degrees_of_freedom))
    cargs.append("[OPTIONS]")
    ret = V1dSemOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file("output.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V1dSemOutputs",
    "V_1D_SEM_METADATA",
    "v_1d_sem",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SWAP_DIMENSIONS_METADATA = Metadata(
    id="6dc09ff3a2acdd106bfb6f85c7da49be2484d100.boutiques",
    name="swap_dimensions",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class SwapDimensionsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `swap_dimensions(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_file_outfile: OutputPathType | None
    """Output name of image, if not provided, writes to standard output."""


def swap_dimensions(
    in_file: InputPathType,
    x_dims_cart: typing.Literal["x", "-x", "y", "-y", "z", "-z"] | None = None,
    x_dims_ras: typing.Literal["LR", "RL", "AP", "PA", "SI", "IS"] | None = None,
    y_dims_ras: typing.Literal["LR", "RL", "AP", "PA", "SI", "IS"] | None = None,
    z_dims_cart: typing.Literal["x", "-x", "y", "-y", "z", "-z"] | None = None,
    z_dims_ras: typing.Literal["LR", "RL", "AP", "PA", "SI", "IS"] | None = None,
    out_file: str | None = None,
    runner: Runner | None = None,
) -> SwapDimensionsOutputs:
    """
    this is an advanced tool that re-orders the data storage to permit changes
    between axial, sagittal and coronal slicing. When used in this mode the same
    left-right convention (also called coordinate handedness or
    radiological/neurological convention) will be maintained as long as no warning
    is printed.
    
    Author: Oxford Centre for Functional MRI of the Brain (FMRIB)
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/Fslutils
    
    Args:
        in_file: Input image to swap dimensions of.
        x_dims_cart: Representation of new x axes in terms of old cartesian\
            axes.
        x_dims_ras: Representation of new x axes in terms of old anatomical\
            axes.
        y_dims_ras: Representation of new y axes in terms of old anatomical\
            axes.
        z_dims_cart: Representation of new z axes in terms of old cartesian\
            axes.
        z_dims_ras: Representation of new z axes in terms of old anatomical\
            axes.
        out_file: Output name of image, if not provided, writes to standard\
            output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SwapDimensionsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SWAP_DIMENSIONS_METADATA)
    cargs = []
    cargs.append("fslswapdim")
    cargs.append(execution.input_file(in_file))
    if x_dims_cart is not None:
        cargs.append(x_dims_cart)
    if x_dims_ras is not None:
        cargs.append(x_dims_ras)
    if y_dims_ras is not None:
        cargs.append(y_dims_ras)
    cargs.append("[B_RAS]")
    if z_dims_cart is not None:
        cargs.append(z_dims_cart)
    if z_dims_ras is not None:
        cargs.append(z_dims_ras)
    if out_file is not None:
        cargs.append(out_file)
    ret = SwapDimensionsOutputs(
        root=execution.output_file("."),
        out_file_outfile=execution.output_file(out_file) if (out_file is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SWAP_DIMENSIONS_METADATA",
    "SwapDimensionsOutputs",
    "swap_dimensions",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__SPHARM_EXAMPLES_METADATA = Metadata(
    id="af6af6754468e947ee22c70cc2ee55297a55529a.boutiques",
    name="@Spharm.examples",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VSpharmExamplesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__spharm_examples(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def v__spharm_examples(
    runner: Runner | None = None,
) -> VSpharmExamplesOutputs:
    """
    A script to demonstrate the usage of spherical harmonics decomposition with
    SUMA.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/@Spharm.examples.html
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VSpharmExamplesOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__SPHARM_EXAMPLES_METADATA)
    cargs = []
    cargs.append("@Spharm.examples")
    cargs.append("[GLOBAL_HELP_OPTION]")
    ret = VSpharmExamplesOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VSpharmExamplesOutputs",
    "V__SPHARM_EXAMPLES_METADATA",
    "v__spharm_examples",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_PERIODOGRAM_METADATA = Metadata(
    id="55241a6d873d3c17ce9b9bd2b05e34541dafe1a3.boutiques",
    name="3dPeriodogram",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dPeriodogramOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_periodogram(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_header: OutputPathType | None
    """Output dataset header file"""
    output_brick: OutputPathType | None
    """Output dataset brick file"""


def v_3d_periodogram(
    dataset: InputPathType,
    prefix: str | None = None,
    taper: float | None = None,
    nfft: float | None = None,
    runner: Runner | None = None,
) -> V3dPeriodogramOutputs:
    """
    Computes the periodogram of each voxel time series. The periodogram is a crude
    estimate of the power spectrum.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dPeriodogram.html
    
    Args:
        dataset: Input dataset.
        prefix: Prefix for the output dataset.
        taper: Fraction of data to taper.
        nfft: Set FFT length to a specific number of points.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dPeriodogramOutputs`).
    """
    if taper is not None and not (0 <= taper <= 1): 
        raise ValueError(f"'taper' must be between 0 <= x <= 1 but was {taper}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_PERIODOGRAM_METADATA)
    cargs = []
    cargs.append("3dPeriodogram")
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    if taper is not None:
        cargs.extend([
            "-taper",
            str(taper)
        ])
    if nfft is not None:
        cargs.extend([
            "-nfft",
            str(nfft)
        ])
    cargs.append(execution.input_file(dataset))
    ret = V3dPeriodogramOutputs(
        root=execution.output_file("."),
        output_header=execution.output_file(prefix + ".HEAD") if (prefix is not None) else None,
        output_brick=execution.output_file(prefix + ".BRIK") if (prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dPeriodogramOutputs",
    "V_3D_PERIODOGRAM_METADATA",
    "v_3d_periodogram",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MAKE_BIANCA_MASK_METADATA = Metadata(
    id="5197c8d961d270d79b99c4f3dfe3006dbc6c90ae.boutiques",
    name="make_bianca_mask",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class MakeBiancaMaskOutputs(typing.NamedTuple):
    """
    Output object returned when calling `make_bianca_mask(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image: OutputPathType
    """Output image"""


def make_bianca_mask(
    runner: Runner | None = None,
) -> MakeBiancaMaskOutputs:
    """
    A script for generating BIANCA masks.
    
    Args:
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MakeBiancaMaskOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MAKE_BIANCA_MASK_METADATA)
    cargs = []
    cargs.append("/usr/local/fsl/bin/make_bianca_mask")
    cargs.append("<Usage:>")
    cargs.append("<Main")
    cargs.append("bet2")
    cargs.append("options>")
    cargs.append("<Variations")
    cargs.append("on")
    cargs.append("default")
    cargs.append("bet2")
    cargs.append("functionality")
    cargs.append("(mutually")
    cargs.append("exclusive")
    cargs.append("options):>")
    cargs.append("<Miscellaneous")
    cargs.append("options:>")
    ret = MakeBiancaMaskOutputs(
        root=execution.output_file("."),
        output_image=execution.output_file("[OUTPUT_IMAGE]"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MAKE_BIANCA_MASK_METADATA",
    "MakeBiancaMaskOutputs",
    "make_bianca_mask",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TOKENS_METADATA = Metadata(
    id="b7eb83c68f44a8e1e5f94df55e43d6e53bb675ee.boutiques",
    name="tokens",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class TokensOutputs(typing.NamedTuple):
    """
    Output object returned when calling `tokens(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def tokens(
    infile: InputPathType | None = None,
    extra_char: list[str] | None = None,
    runner: Runner | None = None,
) -> TokensOutputs:
    """
    Token counting tool.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/tokens.html
    
    Args:
        infile: Specify input file (stdin if none).
        extra_char: Specify extra character to count as valid. Can be used more\
            than once.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TokensOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TOKENS_METADATA)
    cargs = []
    cargs.append("tokens")
    if infile is not None:
        cargs.extend([
            "-infile",
            execution.input_file(infile)
        ])
    if extra_char is not None:
        cargs.extend([
            "-extra",
            *extra_char
        ])
    ret = TokensOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TOKENS_METADATA",
    "TokensOutputs",
    "tokens",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

INSPEC_METADATA = Metadata(
    id="8b4982198616126758279eb8952fb78096cd1ed7.boutiques",
    name="inspec",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class InspecOutputs(typing.NamedTuple):
    """
    Output object returned when calling `inspec(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def inspec(
    specfile: InputPathType,
    newspecname: str | None = None,
    detail: float | None = None,
    leftspec: InputPathType | None = None,
    rightspec: InputPathType | None = None,
    state_rm: str | None = None,
    help_: bool = False,
    runner: Runner | None = None,
) -> InspecOutputs:
    """
    Outputs information found from specfile.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/inspec.html
    
    Args:
        specfile: Spec file to be read.
        newspecname: Rewrite spec file.
        detail: Level of output detail. Default is 1 in general, 0 with\
            -LRmerge. Available levels are 0, 1, 2, and 3.
        leftspec: Merge two spec files in a way that makes sense for viewing in\
            SUMA.
        rightspec: Merge two spec files in a way that makes sense for viewing\
            in SUMA.
        state_rm: Get rid of state STATE_RM from the specfile.
        help_: Display help message.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `InspecOutputs`).
    """
    if detail is not None and not (0 <= detail <= 3): 
        raise ValueError(f"'detail' must be between 0 <= x <= 3 but was {detail}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(INSPEC_METADATA)
    cargs = []
    cargs.append("inspec")
    cargs.extend([
        "-spec",
        execution.input_file(specfile)
    ])
    if newspecname is not None:
        cargs.extend([
            "-prefix",
            newspecname
        ])
    if detail is not None:
        cargs.extend([
            "-detail",
            str(detail)
        ])
    if leftspec is not None:
        cargs.extend([
            "-LRmerge",
            execution.input_file(leftspec)
        ])
    if rightspec is not None:
        cargs.extend([
            "-LRmerge",
            execution.input_file(rightspec)
        ])
    if state_rm is not None:
        cargs.extend([
            "-remove_state",
            state_rm
        ])
    if help_:
        cargs.append("-help")
    ret = InspecOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "INSPEC_METADATA",
    "InspecOutputs",
    "inspec",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

ICA_AROMA_METADATA = Metadata(
    id="2224ad27ba5a247f92986cba9c81a3a71f1d137d.boutiques",
    name="ica_aroma",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class IcaAromaOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ica_aroma(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    aggr_denoised_file: OutputPathType
    """If generated: aggressively denoised volume."""
    nonaggr_denoised_file: OutputPathType
    """If generated: non aggressively denoised volume."""
    out_dir_outfile: OutputPathType
    """file or string representing an existing directory. Directory contains (in
    addition to the denoised files): melodic.ica + classified_motion_components
    + classification_overview + feature_scores + melodic_ic_mni)."""


def ica_aroma(
    feat_dir: InputPathType,
    in_file: InputPathType,
    motion_parameters: InputPathType,
    out_dir: InputPathType,
    tr: float | None = None,
    denoise_type: typing.Literal["nonaggr", "aggr", "both", "no"] = "nonaggr",
    dim: int | None = None,
    fnirt_warp_file: InputPathType | None = None,
    mask: InputPathType | None = None,
    mat_file: InputPathType | None = None,
    melodic_dir: InputPathType | None = None,
    runner: Runner | None = None,
) -> IcaAromaOutputs:
    """
    
    Interface for the ICA_AROMA.py script.
    ICA-AROMA (i.e. 'ICA-based Automatic Removal Of Motion Artifacts') concerns
    a data-driven method to identify and remove motion-related independent
    components from fMRI data. To that end it exploits a small, but robust set
    of theoretically motivated features, preventing the need for classifier
    re-training and therefore providing direct and easy applicability.
    
    Author: Nipype (interface)
    
    URL: https://github.com/rhr-pruim/ICA-AROMA
    
    Args:
        feat_dir: file or string representing an existing directory. If a feat\
            directory exists and temporal filtering has not been run yet, ica_aroma\
            can use the files in this directory.
        in_file: Volume to be denoised.
        motion_parameters: Motion parameters file.
        out_dir: file or string representing a directory. Output directory.
        tr: Tr in seconds. if this is not specified the tr will be extracted\
            from the header of the fmri nifti file.
        denoise_type: 'nonaggr' or 'aggr' or 'both' or 'no'. Type of denoising\
            strategy:-no: only classification, no denoising-nonaggr (default):\
            non-aggresssive denoising, i.e. partial component regression-aggr:\
            aggressive denoising, i.e. full component regression-both: both\
            aggressive and non-aggressive denoising (two outputs).
        dim: Dimensionality reduction when running melodic (default is\
            automatic estimation).
        fnirt_warp_file: File name of the warp-file describing the non-linear\
            registration (e.g. fsl fnirt) of the structural data to mni152 space\
            (.nii.gz).
        mask: Path/name volume mask.
        mat_file: Path/name of the mat-file describing the affine registration\
            (e.g. fsl flirt) of the functional data to structural space (.mat\
            file).
        melodic_dir: file or string representing an existing directory. Path to\
            melodic directory if melodic has already been run.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `IcaAromaOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ICA_AROMA_METADATA)
    cargs = []
    cargs.append("ICA_AROMA")
    if tr is not None:
        cargs.extend([
            "-tr",
            str(tr)
        ])
    cargs.extend([
        "-den",
        denoise_type
    ])
    if dim is not None:
        cargs.extend([
            "-dim",
            str(dim)
        ])
    cargs.extend([
        "-feat",
        execution.input_file(feat_dir)
    ])
    if fnirt_warp_file is not None:
        cargs.extend([
            "-warp",
            execution.input_file(fnirt_warp_file)
        ])
    cargs.extend([
        "-i",
        execution.input_file(in_file)
    ])
    if mask is not None:
        cargs.extend([
            "-m",
            execution.input_file(mask)
        ])
    if mat_file is not None:
        cargs.extend([
            "-affmat",
            execution.input_file(mat_file)
        ])
    if melodic_dir is not None:
        cargs.extend([
            "-meldir",
            execution.input_file(melodic_dir)
        ])
    cargs.extend([
        "-mc",
        execution.input_file(motion_parameters)
    ])
    cargs.extend([
        "-o",
        execution.input_file(out_dir)
    ])
    ret = IcaAromaOutputs(
        root=execution.output_file("."),
        aggr_denoised_file=execution.output_file("out\denoised_func_data_aggr.nii.gz"),
        nonaggr_denoised_file=execution.output_file("out\denoised_func_data_nonaggr.nii.gz"),
        out_dir_outfile=execution.output_file("out"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "ICA_AROMA_METADATA",
    "IcaAromaOutputs",
    "ica_aroma",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CIFTI_ROIS_FROM_EXTREMA_METADATA = Metadata(
    id="0b54d0b3137e3cf72ea0d8a305aec68166794966.boutiques",
    name="cifti-rois-from-extrema",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class CiftiRoisFromExtremaGaussian:
    """
    generate gaussian kernels instead of flat ROIs.
    """
    surf_sigma: float
    """the sigma for the surface gaussian kernel, in mm"""
    vol_sigma: float
    """the sigma for the volume gaussian kernel, in mm"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-gaussian")
        cargs.append(str(self.surf_sigma))
        cargs.append(str(self.vol_sigma))
        return cargs


class CiftiRoisFromExtremaOutputs(typing.NamedTuple):
    """
    Output object returned when calling `cifti_rois_from_extrema(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti"""


def cifti_rois_from_extrema(
    cifti: InputPathType,
    surf_limit: float,
    vol_limit: float,
    direction: str,
    cifti_out: str,
    opt_left_surface_surface: InputPathType | None = None,
    opt_right_surface_surface: InputPathType | None = None,
    opt_cerebellum_surface_surface: InputPathType | None = None,
    gaussian: CiftiRoisFromExtremaGaussian | None = None,
    opt_overlap_logic_method: str | None = None,
    opt_merged_volume: bool = False,
    runner: Runner | None = None,
) -> CiftiRoisFromExtremaOutputs:
    """
    Create cifti roi maps from extrema maps.
    
    For each nonzero value in each map, make a map with an ROI around that
    location. If the -gaussian option is specified, then normalized gaussian
    kernels are output instead of ROIs. The <method> argument to -overlap-logic
    must be one of ALLOW, CLOSEST, or EXCLUDE. ALLOW is the default, and means
    that ROIs are treated independently and may overlap. CLOSEST means that ROIs
    may not overlap, and that no ROI contains vertices that are closer to a
    different seed vertex. EXCLUDE means that ROIs may not overlap, and that any
    vertex within range of more than one ROI does not belong to any ROI.
    
    Author: Washington University School of Medicin
    
    Args:
        cifti: the input cifti.
        surf_limit: geodesic distance limit from vertex, in mm.
        vol_limit: euclidean distance limit from voxel center, in mm.
        direction: which dimension an extrema map is along, ROW or COLUMN.
        cifti_out: the output cifti.
        opt_left_surface_surface: specify the left surface to use: the left\
            surface file.
        opt_right_surface_surface: specify the right surface to use: the right\
            surface file.
        opt_cerebellum_surface_surface: specify the cerebellum surface to use:\
            the cerebellum surface file.
        gaussian: generate gaussian kernels instead of flat ROIs.
        opt_overlap_logic_method: how to handle overlapping ROIs, default\
            ALLOW: the method of resolving overlaps.
        opt_merged_volume: treat volume components as if they were a single\
            component.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiRoisFromExtremaOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_ROIS_FROM_EXTREMA_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-rois-from-extrema")
    cargs.append(execution.input_file(cifti))
    cargs.append(str(surf_limit))
    cargs.append(str(vol_limit))
    cargs.append(direction)
    cargs.append(cifti_out)
    if opt_left_surface_surface is not None:
        cargs.extend([
            "-left-surface",
            execution.input_file(opt_left_surface_surface)
        ])
    if opt_right_surface_surface is not None:
        cargs.extend([
            "-right-surface",
            execution.input_file(opt_right_surface_surface)
        ])
    if opt_cerebellum_surface_surface is not None:
        cargs.extend([
            "-cerebellum-surface",
            execution.input_file(opt_cerebellum_surface_surface)
        ])
    if gaussian is not None:
        cargs.extend(gaussian.run(execution))
    if opt_overlap_logic_method is not None:
        cargs.extend([
            "-overlap-logic",
            opt_overlap_logic_method
        ])
    if opt_merged_volume:
        cargs.append("-merged-volume")
    ret = CiftiRoisFromExtremaOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(cifti_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CIFTI_ROIS_FROM_EXTREMA_METADATA",
    "CiftiRoisFromExtremaGaussian",
    "CiftiRoisFromExtremaOutputs",
    "cifti_rois_from_extrema",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__DJUNCT_EDGY_ALIGN_CHECK_METADATA = Metadata(
    id="c032b5b574d793a5fef13dcf90b64d32e86d71c1.boutiques",
    name="@djunct_edgy_align_check",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VDjunctEdgyAlignCheckOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__djunct_edgy_align_check(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def v__djunct_edgy_align_check(
    ulay: str,
    olay: str,
    prefix: str,
    set_dicom_xyz: list[float] | None = None,
    box_focus_slices: str | None = None,
    montgap: float | None = None,
    montcolor: str | None = None,
    cbar: str | None = None,
    save_ftype: str | None = None,
    umin_fac: float | None = None,
    montx: float | None = None,
    monty: float | None = None,
    use_olay_grid: str | None = None,
    label_mode: str | None = None,
    ulay_range: list[float] | None = None,
    ulay_range_nz: list[float] | None = None,
    ulay_range_am: list[float] | None = None,
    runner: Runner | None = None,
) -> VDjunctEdgyAlignCheckOutputs:
    """
    Helper script for various tasks, heavily modeled on RW Cox's '@snapshot_volreg'
    script.
    
    Author: AFNI Team
    
    URL:
    https://afni.nimh.nih.gov/pub/dist/doc/program_help/@djunct_edgy_align_check.html
    
    Args:
        ulay: ULAY dataset.
        olay: OLAY dataset.
        prefix: Prefix for output files.
        set_dicom_xyz: DICOM coordinates {XX YY ZZ}.
        box_focus_slices: Dataset to focus slices.
        montgap: Gap between slices in montage.
        montcolor: Color for montage.
        cbar: Color bar for overlay.
        save_ftype: File type to save.
        umin_fac: Scaling factor for underlay minimum.
        montx: Number of slices in X-direction for montage.
        monty: Number of slices in Y-direction for montage.
        use_olay_grid: Grid interpolation method for overlay.
        label_mode: Mode for labeling.
        ulay_range: Range for underlay {umin umax}.
        ulay_range_nz: Range for non-zero underlay {umin umax}.
        ulay_range_am: Range for auto-mask underlay {umin umax}.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VDjunctEdgyAlignCheckOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__DJUNCT_EDGY_ALIGN_CHECK_METADATA)
    cargs = []
    cargs.append("@djunct_edgy_align_check")
    cargs.append(ulay)
    cargs.append(olay)
    cargs.append(prefix)
    if set_dicom_xyz is not None:
        cargs.extend(map(str, set_dicom_xyz))
    if box_focus_slices is not None:
        cargs.append(box_focus_slices)
    if montgap is not None:
        cargs.append(str(montgap))
    if montcolor is not None:
        cargs.append(montcolor)
    if cbar is not None:
        cargs.append(cbar)
    if save_ftype is not None:
        cargs.append(save_ftype)
    if umin_fac is not None:
        cargs.append(str(umin_fac))
    if montx is not None:
        cargs.append(str(montx))
    if monty is not None:
        cargs.append(str(monty))
    if use_olay_grid is not None:
        cargs.append(use_olay_grid)
    if label_mode is not None:
        cargs.append(label_mode)
    cargs.append("[help_flag]")
    cargs.append("[ver_flag]")
    cargs.append("[echo_flag]")
    cargs.append("[sharpen_ulay_off_flag]")
    cargs.append("[mask_olay_edges_flag]")
    cargs.append("[no_cor_flag]")
    cargs.append("[no_sag_flag]")
    cargs.append("[no_axi_flag]")
    cargs.append("[no_clean_flag]")
    if ulay_range is not None:
        cargs.extend(map(str, ulay_range))
    if ulay_range_nz is not None:
        cargs.extend(map(str, ulay_range_nz))
    if ulay_range_am is not None:
        cargs.extend(map(str, ulay_range_am))
    ret = VDjunctEdgyAlignCheckOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VDjunctEdgyAlignCheckOutputs",
    "V__DJUNCT_EDGY_ALIGN_CHECK_METADATA",
    "v__djunct_edgy_align_check",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

PROMPT_USER_METADATA = Metadata(
    id="d5cc0d98febf2b18fee05a2473b234e097968d34.boutiques",
    name="prompt_user",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class PromptUserOutputs(typing.NamedTuple):
    """
    Output object returned when calling `prompt_user(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def prompt_user(
    pause_message: str,
    timeout_alias: float | None = None,
    runner: Runner | None = None,
) -> PromptUserOutputs:
    """
    Tool that prompts a window requesting user input with a custom message.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/prompt_user.html
    
    Args:
        pause_message: Pops a window prompting the user with MESSAGE. If\
            MESSAGE is '-', it is read from stdin.
        timeout_alias: Alias for -timeout.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `PromptUserOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(PROMPT_USER_METADATA)
    cargs = []
    cargs.append("prompt_user")
    cargs.extend([
        "<-pause>",
        pause_message
    ])
    if timeout_alias is not None:
        cargs.extend([
            "-to",
            str(timeout_alias)
        ])
    ret = PromptUserOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "PROMPT_USER_METADATA",
    "PromptUserOutputs",
    "prompt_user",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_UNDUMP_METADATA = Metadata(
    id="739057a97effc44c38e6c2e089efe987f53b5056.boutiques",
    name="3dUndump",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dUndumpOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_undump(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfile: OutputPathType | None
    """Main output dataset"""


def v_3d_undump(
    input_files: list[InputPathType],
    prefix: str | None = None,
    master: InputPathType | None = None,
    dimensions: list[float] | None = None,
    mask: InputPathType | None = None,
    datatype: str | None = None,
    dval: float | None = None,
    fval: float | None = None,
    xyz: bool = False,
    sphere_radius: float | None = None,
    cube_mode: bool = False,
    orient: str | None = None,
    head_only: bool = False,
    roimask: InputPathType | None = None,
    allow_nan: bool = False,
    runner: Runner | None = None,
) -> V3dUndumpOutputs:
    """
    Assembles a 3D dataset from an ASCII list of coordinates and optionally values.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dUndump.html
    
    Args:
        input_files: Input ASCII file(s), with one voxel specification per\
            line.
        prefix: 'ppp' is the prefix for the output dataset [default = undump].
        master: 'mmm' is the master dataset, whose geometry will determine the\
            geometry of the output.
        dimensions: Sets the dimensions of the output dataset to be I by J by K\
            voxels.
        mask: Specifies a mask dataset 'MMM', which will control which voxels\
            are allowed to get values set.
        datatype: 'type' determines the voxel data type of the output, which\
            may be byte, short, or float [default = short].
        dval: 'vvv' is the default value stored in each input voxel that does\
            not have a value supplied in the input file [default = 1].
        fval: 'fff' is the fill value, used for each voxel in the output\
            dataset that is NOT listed in the input file [default = 0].
        xyz: Coordinates in the input file are (x,y,z) spatial coordinates, in\
            mm.
        sphere_radius: Specifies that a sphere of radius 'rrr' will be filled\
            about each input (x,y,z) or (i,j,k) voxel.
        cube_mode: Put cubes down instead of spheres. The 'radius' then is half\
            the length of a side.
        orient: Specifies the coordinate order used by -xyz. The code must be 3\
            letters, one each from the pairs {R,L} {A,P} {I,S}.
        head_only: Creates only the .HEAD file.
        roimask: Specifies which voxels get what numbers by using a dataset\
            'rrr', instead of coordinates.
        allow_nan: Allow NaN (not-a-number) values to be entered.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dUndumpOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_UNDUMP_METADATA)
    cargs = []
    cargs.append("3dUndump")
    cargs.extend([execution.input_file(f) for f in input_files])
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    if master is not None:
        cargs.extend([
            "-master",
            execution.input_file(master)
        ])
    if dimensions is not None:
        cargs.extend([
            "-dimen",
            *map(str, dimensions)
        ])
    if mask is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask)
        ])
    if datatype is not None:
        cargs.extend([
            "-datum",
            datatype
        ])
    if dval is not None:
        cargs.extend([
            "-dval",
            str(dval)
        ])
    if fval is not None:
        cargs.extend([
            "-fval",
            str(fval)
        ])
    if xyz:
        cargs.append("-xyz")
    if sphere_radius is not None:
        cargs.extend([
            "-srad",
            str(sphere_radius)
        ])
    if cube_mode:
        cargs.append("-cubes")
    if orient is not None:
        cargs.extend([
            "-orient",
            orient
        ])
    if head_only:
        cargs.append("-head_only")
    if roimask is not None:
        cargs.extend([
            "-ROImask",
            execution.input_file(roimask)
        ])
    if allow_nan:
        cargs.append("-allow_NaN")
    ret = V3dUndumpOutputs(
        root=execution.output_file("."),
        outfile=execution.output_file(prefix + ".nii.gz") if (prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dUndumpOutputs",
    "V_3D_UNDUMP_METADATA",
    "v_3d_undump",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

UN_WARP_EPI_PY_METADATA = Metadata(
    id="108a1f73f3e184e54f69ea8ebe47acc68251fc95.boutiques",
    name="unWarpEPI.py",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class UnWarpEpiPyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `un_warp_epi_py(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def un_warp_epi_py(
    forward: InputPathType,
    reverse: InputPathType,
    anat4warp: InputPathType,
    data: str,
    subj_id: str,
    giant_move: bool = False,
    runner: Runner | None = None,
) -> UnWarpEpiPyOutputs:
    """
    Routine to unwarp EPI data set using another data set with opposite polarity.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/unWarpEPI.py.html
    
    Args:
        forward: Calibration matching data to be corrected.
        reverse: Calibration with opposing polarity to data to be corrected.
        anat4warp: Reference anatomical dataset.
        data: Data to be corrected (same polarity as forward calibration data).\
            Separate with commas if specifying multiple datasets.
        subj_id: ID of subject to be corrected.
        giant_move: Set giant_move option for align_epi_anat if final align of\
            anatomy to corrected EPI fails if datasets are far apart in space.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `UnWarpEpiPyOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(UN_WARP_EPI_PY_METADATA)
    cargs = []
    cargs.append("unWarpEPI.py")
    cargs.append("-f")
    cargs.extend([
        "-f",
        execution.input_file(forward)
    ])
    cargs.append("-r")
    cargs.extend([
        "-r",
        execution.input_file(reverse)
    ])
    cargs.append("-a")
    cargs.extend([
        "-a",
        execution.input_file(anat4warp)
    ])
    cargs.append("-d")
    cargs.extend([
        "-d",
        data
    ])
    cargs.append("-s")
    cargs.extend([
        "-s",
        subj_id
    ])
    if giant_move:
        cargs.append("-g")
    ret = UnWarpEpiPyOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "UN_WARP_EPI_PY_METADATA",
    "UnWarpEpiPyOutputs",
    "un_warp_epi_py",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

UBER_PROC_PY_METADATA = Metadata(
    id="36e6d5db2ea98de96052bdf63be621bc71fbff06.boutiques",
    name="uber_proc.py",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class UberProcPyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `uber_proc_py(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def uber_proc_py(
    results_dir: str | None = None,
    runner: Runner | None = None,
) -> UberProcPyOutputs:
    """
    Uber processing tool - work in progress.
    
    Author: AFNI Team
    
    URL: https://afni.nimh.nih.gov/pub/dist/doc/program_help/uber_proc.py.html
    
    Args:
        results_dir: Directory where results will be placed. Default is a new\
            'uber_results' directory in the current working directory.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `UberProcPyOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(UBER_PROC_PY_METADATA)
    cargs = []
    cargs.append("uber_proc.py")
    if results_dir is not None:
        cargs.append(results_dir)
    ret = UberProcPyOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "UBER_PROC_PY_METADATA",
    "UberProcPyOutputs",
    "uber_proc_py",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

ZEROPAD_METADATA = Metadata(
    id="dc7ff5df7b78374a3e83684ba9e7a5734ffacc12.boutiques",
    name="zeropad",
    package="fsl",
    container_image_tag="mcin/fsl:6.0.5",
)


class ZeropadOutputs(typing.NamedTuple):
    """
    Output object returned when calling `zeropad(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """File containing the zero-padded output"""


def zeropad(
    input_number: str,
    length: float,
    runner: Runner | None = None,
) -> ZeropadOutputs:
    """
    Tool for zero-padding numbers to a specified length.
    
    Author: Unknown
    
    URL: Unknown
    
    Args:
        input_number: Input number to be zero-padded.
        length: Desired length of the output string.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ZeropadOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(ZEROPAD_METADATA)
    cargs = []
    cargs.append("zeropad")
    cargs.append(input_number)
    cargs.append(str(length))
    ret = ZeropadOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file("padded_output.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "ZEROPAD_METADATA",
    "ZeropadOutputs",
    "zeropad",
]
